/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.io.*;
import java.util.*;
import EDU.auburn.VGJ.graph.Node;
import EDU.utwente.CoCompose.lang.*;
import EDU.utwente.CoCompose.translate.*;
import EDU.utwente.CoCompose.util.CoComposeException;

/**
 * A dialog window for editing the implementations list of Concepts.
 *
 * @author Dennis Wagelaar
 * @version 0.1.05, 11 December 2002
 */

public class ImplementationsDialog extends CoComposeDialog
{
  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  private Concept concept_;
  private Vector implementations_ = new Vector();
  private Implementation currentImpl_ = new Implementation();
  private Vector constraints_ = new Vector();
  private Vector constraintConcepts_ = new Vector();
  JPanel panel1 = new JPanel();
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel implementationsPanel = new JPanel();
  JPanel buttonPanel = new JPanel();
  JButton okBtn = new JButton();
  FlowLayout flowLayout1 = new FlowLayout();
  JLabel jLabel1 = new JLabel();
  JPanel languagePanel = new JPanel();
  BorderLayout borderLayout2 = new BorderLayout();
  JPanel implementationPanel = new JPanel();
  JPanel filenamePanel = new JPanel();
  BorderLayout borderLayout3 = new BorderLayout();
  JButton browseBtn = new JButton();
  JPanel implementationPanel2 = new JPanel();
  BorderLayout borderLayout4 = new BorderLayout();
  JPanel addRemovePanel = new JPanel();
  JButton addBtn = new JButton();
  JButton removeBtn = new JButton();
  JTextField languageEdit = new JTextField();
  FlowLayout flowLayout2 = new FlowLayout();
  FlowLayout flowLayout3 = new FlowLayout();
  FlowLayout flowLayout4 = new FlowLayout();
  JPanel listPanel = new JPanel();
  FlowLayout flowLayout5 = new FlowLayout();
  JScrollPane jScrollPane1 = new JScrollPane();
  JLabel jLabel3 = new JLabel();
  JList implementationsList = new JList();
  JButton upBtn = new JButton();
  JButton downBtn = new JButton();
  JPanel implementationPanel4 = new JPanel();
  BorderLayout borderLayout5 = new BorderLayout();
  JLabel jLabel4 = new JLabel();
  FlowLayout flowLayout6 = new FlowLayout();
  JTextField formEdit = new JTextField();
  JPanel formPanel = new JPanel();
  JPanel implementationPanel5 = new JPanel();
  BorderLayout borderLayout6 = new BorderLayout();
  JPanel contentPanel = new JPanel();
  FlowLayout flowLayout7 = new FlowLayout();
  JLabel jLabel5 = new JLabel();
  JScrollPane jScrollPane2 = new JScrollPane();
  JTextPane contentPane = new JTextPane();
  JPanel implementationsPanel1 = new JPanel();
  BorderLayout borderLayout7 = new BorderLayout();
  JPanel constraintsPanel = new JPanel();
  BorderLayout borderLayout8 = new BorderLayout();
  JPanel constraintsPanel1 = new JPanel();
  JLabel jLabel6 = new JLabel();
  Border border1;
  TitledBorder titledBorder1;
  Border border2;
  TitledBorder titledBorder2;
  Border border3;
  JPanel implementationPanel3 = new JPanel();
  BorderLayout borderLayout9 = new BorderLayout();
  Border border4;
  Border border5;
  TitledBorder titledBorder3;
  JPanel implementationsPanel2 = new JPanel();
  Border border6;
  TitledBorder titledBorder4;
  BorderLayout borderLayout10 = new BorderLayout();
  JPanel constraintsPanel2 = new JPanel();
  JPanel constraintsPanel3 = new JPanel();
  FlowLayout flowLayout8 = new FlowLayout();
  BorderLayout borderLayout11 = new BorderLayout();
  JLabel jLabel7 = new JLabel();
  JTextField constraintFormEdit = new JTextField();
  JPanel constraintsPanel4 = new JPanel();
  BorderLayout borderLayout12 = new BorderLayout();
  JPanel constraintsPanel5 = new JPanel();
  FlowLayout flowLayout9 = new FlowLayout();
  JButton addConstraintBtn = new JButton();
  JButton removeConstraintBtn = new JButton();
  JPanel constraintsPanel6 = new JPanel();
  FlowLayout flowLayout10 = new FlowLayout();
  JLabel jLabel8 = new JLabel();
  JList constraintsList = new JList();
  JScrollPane jScrollPane3 = new JScrollPane();
  JButton clearConstraintsBtn = new JButton();
  JButton updateBtn = new JButton();
  JComboBox constraintConceptBox = new JComboBox();

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs an ImplementationsDialog for a Concept object.
   *
   * @param frame The parent frame.
   * @param title The dialog title.
   * @param modal Shows the dialog in modal mode when true.
   * @param concept The Concept of which the implementations will be edited.
   */
  public ImplementationsDialog(Frame frame, String title, boolean modal,
    Concept concept)
  {
    super(frame, title, modal);
    concept_ = concept;
    try
    {
      jbInit();
      pack();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  } //ImplementationsDialog(Frame frame, String title, boolean modal, Concept concept)

  /**
   * Constructs an ImplementationsDialog.
   * This constructor is only for the JBuilder Designer, do not use it!
   */
  ImplementationsDialog()
  {
    this(null, "", false, null);
  } //ImplementationsDialog()

  //////////////////////////////////////////////////////////////////////////////
  // Private methods
  //////////////////////////////////////////////////////////////////////////////

  /* JBuilder Component initialization. */
  void jbInit() throws Exception
  {
    border1 = BorderFactory.createEtchedBorder(Color.white,new Color(148, 145, 140));
    titledBorder1 = new TitledBorder(border1,"Constraints");
    border2 = BorderFactory.createEtchedBorder(Color.white,new Color(148, 145, 140));
    titledBorder2 = new TitledBorder(border2,"Implementations");
    border3 = BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(SystemColor.controlText,1),BorderFactory.createEmptyBorder(5,5,5,5));
    border4 = BorderFactory.createEmptyBorder(5,5,5,5);
    border5 = BorderFactory.createEtchedBorder(Color.white,new Color(148, 145, 140));
    titledBorder3 = new TitledBorder(border5,"Implementation");
    border6 = BorderFactory.createEtchedBorder(Color.white,new Color(148, 145, 140));
    titledBorder4 = new TitledBorder(border6,"Implementations");
    panel1.setLayout(borderLayout1);
    okBtn.setText("OK");
    okBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        okBtn_actionPerformed(e);
      }
    });
    buttonPanel.setLayout(flowLayout1);
    jLabel1.setMaximumSize(new Dimension(200, 17));
    jLabel1.setMinimumSize(new Dimension(200, 17));
    jLabel1.setPreferredSize(new Dimension(200, 17));
    jLabel1.setText("Language:");
    implementationsPanel.setLayout(borderLayout2);
    implementationPanel.setLayout(borderLayout3);
    browseBtn.setText("Browse for implementation file");
    browseBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        browseBtn_actionPerformed(e);
      }
    });
    implementationPanel2.setLayout(borderLayout4);
    addBtn.setText("Add");
    addBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        addBtn_actionPerformed(e);
      }
    });
    removeBtn.setText("Remove");
    removeBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        removeBtn_actionPerformed(e);
      }
    });
    languageEdit.setFont(new java.awt.Font("Dialog", 0, 12));
    languageEdit.setPreferredSize(new Dimension(400, 21));
    languagePanel.setLayout(flowLayout3);
    constraintConceptBox.setBorder(languageEdit.getBorder());
    constraintConceptBox.setMaximumSize(new Dimension(32767, 24));
    constraintConceptBox.setMinimumSize(new Dimension(126, 24));
    constraintConceptBox.setPreferredSize(new Dimension(400, 24));
    languagePanel.add(jLabel1, null);
    languagePanel.add(languageEdit, null);
    implementationPanel.add(languagePanel, BorderLayout.NORTH);
    filenamePanel.setLayout(flowLayout2);
    addRemovePanel.setLayout(flowLayout4);
    listPanel.setLayout(flowLayout5);
    jScrollPane1.setBorder(languageEdit.getBorder());
    jScrollPane1.setPreferredSize(new Dimension(400, 100));
    jLabel3.setMaximumSize(new Dimension(200, 17));
    jLabel3.setMinimumSize(new Dimension(200, 17));
    jLabel3.setPreferredSize(new Dimension(200, 17));
    jLabel3.setText("Installed implementations:");
    implementationsList.addListSelectionListener(new javax.swing.event.ListSelectionListener()
    {

      public void valueChanged(ListSelectionEvent e)
      {
        implementationsList_valueChanged(e);
      }
    });
    upBtn.setText("Move up");
    upBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        upBtn_actionPerformed(e);
      }
    });
    downBtn.setText("Move down");
    downBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        downBtn_actionPerformed(e);
      }
    });
    panel1.setBorder(BorderFactory.createEtchedBorder());
    implementationPanel4.setLayout(borderLayout5);
    jLabel4.setText("Concept form:");
    jLabel4.setPreferredSize(new Dimension(200, 17));
    jLabel4.setMinimumSize(new Dimension(200, 17));
    jLabel4.setMaximumSize(new Dimension(200, 17));
    formEdit.setPreferredSize(new Dimension(400, 21));
    formEdit.setFont(new java.awt.Font("Dialog", 0, 12));
    formPanel.setLayout(flowLayout6);
    implementationPanel5.setLayout(borderLayout6);
    contentPanel.setLayout(flowLayout7);
    jLabel5.setMaximumSize(new Dimension(200, 17));
    jLabel5.setMinimumSize(new Dimension(200, 17));
    jLabel5.setPreferredSize(new Dimension(200, 17));
    jLabel5.setText("Implementation content:");
    jScrollPane2.setBorder(languageEdit.getBorder());
    jScrollPane2.setPreferredSize(new Dimension(400, 96));
    implementationsPanel1.setLayout(borderLayout7);
    constraintsPanel.setLayout(borderLayout8);
    jLabel6.setMaximumSize(new Dimension(200, 17));
    jLabel6.setMinimumSize(new Dimension(200, 17));
    jLabel6.setPreferredSize(new Dimension(200, 17));
    jLabel6.setText("Concept:");
    constraintsPanel.setBorder(titledBorder1);
    implementationPanel3.setLayout(borderLayout9);
    implementationPanel.setBorder(titledBorder3);
    implementationsPanel2.setBorder(titledBorder4);
    implementationsPanel2.setLayout(borderLayout10);
    constraintsPanel1.setLayout(flowLayout8);
    constraintsPanel2.setLayout(borderLayout11);
    jLabel7.setText("Equals concept form:");
    jLabel7.setPreferredSize(new Dimension(200, 17));
    jLabel7.setMinimumSize(new Dimension(200, 17));
    jLabel7.setMaximumSize(new Dimension(200, 17));
    constraintFormEdit.setPreferredSize(new Dimension(400, 21));
    constraintFormEdit.setFont(new java.awt.Font("Dialog", 0, 12));
    constraintsPanel4.setLayout(borderLayout12);
    constraintsPanel5.setLayout(flowLayout9);
    addConstraintBtn.setText("Set");
    addConstraintBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        addConstraintBtn_actionPerformed(e);
      }
    });
    removeConstraintBtn.setText("Remove");
    removeConstraintBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        removeConstraintBtn_actionPerformed(e);
      }
    });
    constraintsPanel6.setLayout(flowLayout10);
    jLabel8.setText("Installed constraints:");
    jLabel8.setPreferredSize(new Dimension(200, 17));
    jLabel8.setMinimumSize(new Dimension(200, 17));
    jLabel8.setMaximumSize(new Dimension(200, 17));
    constraintsList.addListSelectionListener(new javax.swing.event.ListSelectionListener()
    {

      public void valueChanged(ListSelectionEvent e)
      {
        constraintsList_valueChanged(e);
      }
    });
    jScrollPane3.setPreferredSize(new Dimension(400, 100));
    jScrollPane3.setBorder(languageEdit.getBorder());
    clearConstraintsBtn.setText("Clear");
    clearConstraintsBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        clearConstraintsBtn_actionPerformed(e);
      }
    });
    constraintsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    implementationsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    updateBtn.setText("Update");
    updateBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        updateBtn_actionPerformed(e);
      }
    });
    getContentPane().add(panel1);
    panel1.add(implementationsPanel, BorderLayout.CENTER);
    implementationsPanel.add(implementationPanel,  BorderLayout.NORTH);
    implementationPanel.add(implementationPanel2,  BorderLayout.CENTER);
    addRemovePanel.add(addBtn, null);
    addRemovePanel.add(updateBtn, null);
    addRemovePanel.add(removeBtn, null);
    addRemovePanel.add(upBtn, null);
    addRemovePanel.add(downBtn, null);
    implementationsPanel2.add(listPanel,  BorderLayout.CENTER);
    implementationsPanel1.add(implementationsPanel2,  BorderLayout.NORTH);
    implementationsPanel2.add(addRemovePanel, BorderLayout.NORTH);
    listPanel.add(jLabel3, null);
    listPanel.add(jScrollPane1, null);
    jScrollPane1.getViewport().add(implementationsList, null);
    formPanel.add(jLabel4, null);
    formPanel.add(formEdit, null);
    implementationPanel3.add(implementationPanel4,  BorderLayout.CENTER);
    panel1.add(buttonPanel, BorderLayout.SOUTH);
    buttonPanel.add(okBtn, null);
    contentPanel.add(jLabel5, null);
    contentPanel.add(jScrollPane2, null);
    implementationPanel3.add(filenamePanel, BorderLayout.NORTH);
    filenamePanel.add(browseBtn, null);
    implementationPanel2.add(formPanel, BorderLayout.NORTH);
    implementationPanel4.add(implementationPanel5, BorderLayout.CENTER);
    implementationPanel5.add(constraintsPanel, BorderLayout.CENTER);
    constraintsPanel.add(constraintsPanel1, BorderLayout.NORTH);
    jScrollPane2.getViewport().add(contentPane, null);
    implementationPanel4.add(contentPanel, BorderLayout.NORTH);
    constraintsPanel1.add(jLabel6, null);
    constraintsPanel1.add(constraintConceptBox, null);
    implementationsPanel.add(implementationsPanel1,  BorderLayout.CENTER);
    implementationPanel2.add(implementationPanel3,  BorderLayout.CENTER);
    constraintsPanel.add(constraintsPanel2, BorderLayout.CENTER);
    constraintsPanel2.add(constraintsPanel3, BorderLayout.NORTH);
    constraintsPanel3.add(jLabel7, null);
    constraintsPanel3.add(constraintFormEdit, null);
    constraintsPanel2.add(constraintsPanel4, BorderLayout.CENTER);
    constraintsPanel4.add(constraintsPanel5,  BorderLayout.NORTH);
    constraintsPanel5.add(addConstraintBtn, null);
    constraintsPanel5.add(removeConstraintBtn, null);
    constraintsPanel5.add(clearConstraintsBtn, null);
    constraintsPanel4.add(constraintsPanel6, BorderLayout.CENTER);
    constraintsPanel6.add(jLabel8, null);
    constraintsPanel6.add(jScrollPane3, null);
    jScrollPane3.getViewport().add(constraintsList, null);
  } //jbInit

  /* Reads the concepts from the model into the combobox. */
  private void readConcepts()
  {
    AbstractConcept concept;
    PublishedConcept pub;
    Model model = concept_.getModel();

    if (model != null)
    {
      constraintConceptBox.removeAllItems();
      constraintConcepts_.clear();
      for (Node thisNode = model.firstNode(); thisNode != null;
        thisNode = model.nextNode(thisNode))
      {
        if ((thisNode instanceof EDU.utwente.CoCompose.lang.Concept) ||
            (thisNode instanceof EDU.utwente.CoCompose.lang.SolutionRole))
        {
          concept = (AbstractConcept) thisNode;
          if ((concept != concept_) && (concept.getName().length() > 0))
          { // add only concepts other than this with name set
            constraintConceptBox.addItem(concept.getName());
            // save in vector too, so we can retrieve the object later on.
            constraintConcepts_.add(concept);
          }
        } //if Concept
        else if (thisNode instanceof EDU.utwente.CoCompose.lang.PublishedConcept)
        {
          pub = (PublishedConcept) thisNode;
          if (pub.getPublicName().length() > 0)
          { // add only concepts other than this with name set
            constraintConceptBox.addItem(pub.getPublicName());
            // save in vector too, so we can retrieve the object later on.
            constraintConcepts_.add(pub);
          }
        } //if PublishedConcept
      } //for
    }
  } //readConcepts

  /* Reads implementations from concept into listbox. */
  private void readImplementations()
  {
    Implementation implementation;
    Vector implementationNames = new Vector();

    implementationsList.removeAll();
    implementations_.clear();
    if (concept_ == null)
      return;

    Enumeration implementations = concept_.getImplementations();
    while (implementations.hasMoreElements())
    {
      implementation = (Implementation) implementations.nextElement();
      implementations_.add(implementation);
      implementationNames.add(implementation.getLanguage() + ", " +
        implementation.getConceptForm());
    }
    implementationsList.setListData(implementationNames);
  } //readImplementations

  /* Reads implementation constraints into listbox. */
  private void readConstraints()
  {
    Implementation implementation = getCurrentImplementation();
    Hashtable constraints = implementation.getConstraints();
    Collection conceptForms;
    Iterator conceptFs;
    String forms;
    Vector constraintNames = new Vector();
    AbstractConcept concept;

    constraintsList.removeAll();
    constraints_.clear();
    Enumeration concepts = constraints.keys();
    while (concepts.hasMoreElements())
    {
      concept = (AbstractConcept) concepts.nextElement();
      constraints_.add(concept);
      conceptForms = (Collection) constraints.get(concept);
      conceptFs = conceptForms.iterator();
      // at least one form
      forms = (String) conceptFs.next();
      while (conceptFs.hasNext())
      {
        forms += ' ' + (String) conceptFs.next();
      }
      if (concept instanceof EDU.utwente.CoCompose.lang.PublishedConcept)
      { // use public name on published concepts
        constraintNames.add(((PublishedConcept) concept).getPublicName() +
          " = " + forms);
      }
      else
      { // use normal name on other concepts
        constraintNames.add(concept.getName() + " = " + forms);
      }
    }
    constraintsList.setListData(constraintNames);
    constraintConceptBox.setSelectedItem(null);
    constraintFormEdit.setText("");
  } //readConstraints

  /* Browse for a file name. */
  void browseBtn_actionPerformed(ActionEvent e)
  {
    // browse for any file
    JFileChooser browseDlg = new JFileChooser("./implementations");
    browseDlg.setDialogTitle("Browse for implementation file");
    if (browseDlg.showOpenDialog(this) != JFileChooser.APPROVE_OPTION)
      return;
    File file = browseDlg.getSelectedFile();
    // read file
    try
    {
      FileInputStream stream = new FileInputStream(file);
      byte buffer[] = new byte[stream.available()]; // create buffer
      int length = stream.read(buffer);
      contentPane.setText(new String(buffer, 0, length));
    }
    catch (Exception exc)
    {
      reportError("Could not read the file: " + exc.getMessage(), "Error", null);
    }
  } //browseBtn_actionPerformed

  /* Adds an implementation to the concept. */
  void addBtn_actionPerformed(ActionEvent e)
  {
    // check if implementation ok
    if ((languageEdit.getText().length() == 0) ||
        (formEdit.getText().length() == 0))
    {
      reportError("Implementation language and concept form must be set",
        "Error", null);
      return;
    }
    // get and edit current implementation
    Implementation implementation = getCurrentImplementation();
    // add implementation
    concept_.addImplementation(implementation);
    readImplementations();
    implementationsList.setSelectedIndex(implementations_.size() - 1);
  } //addBtn_actionPerformed

  /* Updates the currently selected implementation. */
  void updateBtn_actionPerformed(ActionEvent e)
  {
    Implementation implementation, currentImpl;

    if (implementationsList.getSelectedIndex() < 0)
      return;

    currentImpl = getCurrentImplementation();
    implementation = (Implementation) implementations_.get(implementationsList.getSelectedIndex());
    concept_.removeImplementation(implementation);
    implementation.copyAttributes(currentImpl);
    try
    {
      implementation.copyConstraints(currentImpl);
    }
    catch (CoComposeException exc)
    {
      exc.printStackTrace();
    }
    concept_.addImplementation(implementation);
    readImplementations();
  } //updateBtn_actionPerformed

  /* Removes an implementation from the concept. */
  void removeBtn_actionPerformed(ActionEvent e)
  {
    Implementation implementation;

    if (implementationsList.getSelectedIndex() < 0)
      return;

    implementation = (Implementation) implementations_.get(implementationsList.getSelectedIndex());
    // remove all dependencies to the model
    implementation.clearConstraints();
    concept_.removeImplementation(implementation);
    readImplementations();
  } //removeBtn_actionPerformed

  /* Closes window. */
  void okBtn_actionPerformed(ActionEvent e)
  {
    dispose();
  } //okBtn_actionPerformed

  /* Other list item selected. */
  void implementationsList_valueChanged(ListSelectionEvent e)
  {
    Implementation implementation, listImpl;

    if (implementationsList.getSelectedIndex() < 0)
      return;

    listImpl = (Implementation)
      implementations_.get(implementationsList.getSelectedIndex());
    implementation = new Implementation();
    implementation.copyAttributes(listImpl);
    try
    {
      implementation.copyConstraints(listImpl);
    }
    catch (CoComposeException exc)
    {
      exc.printStackTrace();
    }
    setCurrentImplementation(implementation);
  } //implementationsList_valueChanged

  /* Move up selected implementation patterm. */
  void upBtn_actionPerformed(ActionEvent e)
  {
    Implementation implementation;
    int index = implementationsList.getSelectedIndex();

    if (index < 1)
      return;

    implementation = (Implementation) implementations_.get(index);
    concept_.removeImplementation(implementation);
    concept_.insertImplementation(implementation, index - 1);
    readImplementations();
    implementationsList.setSelectedIndex(index - 1);
  } //upBtn_actionPerformed

  /* Move down selected implementation patterm. */
  void downBtn_actionPerformed(ActionEvent e)
  {
    Implementation implementation;
    int index = implementationsList.getSelectedIndex();

    if ((index < 0) || (index >= implementations_.size() - 1))
      return;

    implementation = (Implementation) implementations_.get(index);
    concept_.removeImplementation(implementation);
    concept_.insertImplementation(implementation, index + 1);
    readImplementations();
    implementationsList.setSelectedIndex(index + 1);
  } //downBtn_actionPerformed

  /* A value from constraintsList is selected. */
  void constraintsList_valueChanged(ListSelectionEvent e)
  {
    Implementation implementation = getCurrentImplementation();
    Hashtable constraints = implementation.getConstraints();
    AbstractConcept concept;
    Collection conceptForms;
    Iterator conceptFs;
    String name, forms;

    if (constraintsList.getSelectedIndex() < 0)
      return;
    concept = (AbstractConcept) constraints_.get(constraintsList.getSelectedIndex());
    // use public name on published concepts, else normal name
    if (concept instanceof EDU.utwente.CoCompose.lang.PublishedConcept)
      name = ((PublishedConcept) concept).getPublicName();
    else
      name = concept.getName();
    conceptForms = (Collection) constraints.get(concept);
    conceptFs = conceptForms.iterator();
    // at least one form
    forms = (String) conceptFs.next();
    while (conceptFs.hasNext())
    {
      forms += ' ' + (String) conceptFs.next();
    }
    constraintConceptBox.setSelectedItem(name);
    constraintFormEdit.setText(forms);
  } //constraintsList_valueChanged

  /* A constraint must be added. */
  void addConstraintBtn_actionPerformed(ActionEvent e)
  {
    Implementation implementation = getCurrentImplementation();
    Model model = concept_.getModel();
    AbstractConcept concept;
    int conceptIndex = constraintConceptBox.getSelectedIndex();
    String forms = constraintFormEdit.getText();
    int sepIndex, prevIndex;
    Collection conceptForms = new Vector();

    // check if implementation data ok
    if ((conceptIndex < 0) || (forms.length() == 0))
    {
      reportError("Constraint concept name and forms must be set", "Error", null);
      return;
    }
    // retrieve concept from list
    concept = (AbstractConcept) constraintConcepts_.get(conceptIndex);
    // check if exists
    if (concept == null)
    { // should never happen
      reportError("Constraint concept not found", "Error", null);
      return;
    }
    // parse forms (space is separator character)
    prevIndex = 0;
    while ((sepIndex = forms.indexOf(' ', prevIndex)) != -1)
    { // add multiple concept forms
      conceptForms.add(forms.substring(prevIndex, sepIndex));
      prevIndex = sepIndex + 1;
    }
    // add last concept form
    conceptForms.add(forms.substring(prevIndex));
    // add new constraint
    implementation.addConstraint(concept, conceptForms);
    readConstraints();
  } //addConstraintBtn_actionPerformed

  /* A constraint must be removed. */
  void removeConstraintBtn_actionPerformed(ActionEvent e)
  {
    AbstractConcept concept;

    if (constraintsList.getSelectedIndex() < 0)
      return;

    Implementation implementation = getCurrentImplementation();
    concept = (AbstractConcept) constraints_.get(constraintsList.getSelectedIndex());
    implementation.removeConstraint(concept);
    readConstraints();
  } //removeConstraintBtn_actionPerformed

  /* The list of constraints must be cleared. */
  void clearConstraintsBtn_actionPerformed(ActionEvent e)
  {
    Implementation implementation = getCurrentImplementation();
    implementation.clearConstraints();
    readConstraints();
  } //clearConstraintsBtn_actionPerformed

  //////////////////////////////////////////////////////////////////////////////
  // Protected methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Refreshes window contents.
   */
  protected void refresh()
  {
    Implementation implementation = getCurrentImplementation();

    // refresh attribute controls
    languageEdit.setText(implementation.getLanguage());
    formEdit.setText(implementation.getConceptForm());
    contentPane.setText(implementation.getContent());
    // re-read constraints
    readConstraints();
  } //refresh

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Gets the implementation that is currently being edited.
   *
   * @return The current implementation.
   */
  public Implementation getCurrentImplementation()
  {
    currentImpl_.setLanguage(languageEdit.getText());
    currentImpl_.setConceptForm(formEdit.getText());
    currentImpl_.setContent(contentPane.getText());
    return currentImpl_;
  } //getCurrentImplementation

  /**
   * Gets the implementation that is currently being edited.
   *
   * @param impl The current implementation.
   */
  public void setCurrentImplementation(Implementation impl)
  {
    languageEdit.setText(impl.getLanguage());
    formEdit.setText(impl.getConceptForm());
    contentPane.setText(impl.getContent());
    currentImpl_ = impl;
    // refresh window
    refresh();
  } //setCurrentImplementation

  /**
   * Shows dialog window.
   */
  public void show()
  {
    // read concepts
    readConcepts();
    // read implementations into list
    readImplementations();
    readConstraints();
    super.show();
  } //show
} //class ImplementationsDialog
