/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.net.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import EDU.utwente.CoCompose.CoCompose;

/**
 * About box window to go with the application. Shows brief information about
 * the tool.
 *
 * @author Dennis Wagelaar
 * @version 0.1.10, 23 September 2003
 */

public class MainFrame_AboutBox extends JDialog implements ActionListener
{
  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  private Frame frame_;
  ImageIcon aboutIcon = new ImageIcon("./images/CoCompose.jpg");
  JPanel panel1 = new JPanel();
  JPanel insetsPanel1 = new JPanel();
  JPanel insetsPanel2 = new JPanel();
  JButton button1 = new JButton();
  BorderLayout borderLayout1 = new BorderLayout();
  FlowLayout flowLayout1 = new FlowLayout();
  FlowLayout flowLayout2 = new FlowLayout();
  String header;
  String product   = "<FONT Size=\"+1\">CoCompose Design Language Tool</FONT>";
  String version   = "<P>Version " + CoCompose.MAJOR_VERSION + "." +
                     CoCompose.MINOR_VERSION + "." + CoCompose.MINOR_SUFFIX +
                     ", " + CoCompose.DATE + "</P>";
  String comments  = "<P>This is a prototype to test and further develop the " +
                     "CoCompose Design Language.</P>";
  String copyright = "<P>Copyright (c) 2002, 2003, Dennis Wagelaar, " +
                     "SSEL Group, Vrije Universiteit Brussel " +
                     "(<A Href=\"http://ssel.vub.ac.be/\">" +
                     "http://ssel.vub.ac.be/</A>), " +
                     "TRESE Group, University of Twente " +
                     "(<A Href=\"http://trese.cs.utwente.nl/\">" +
                     "http://trese.cs.utwente.nl/</A>)</P>" +
                     "<P>This program is free software; you can redistribute " +
                     "it and/or modify it under the terms of the GNU General " +
                     "Public License, version 2 as published by the Free " +
                     "Software Foundation.</P>" +
                     "<P>This program is distributed in the hope that it " +
                     "will be useful, but WITHOUT ANY WARRANTY; without even " +
                     "the implied warranty of MERCHANTABILITY or FITNESS FOR " +
                     "A PARTICULAR PURPOSE.  See the <A Href=\"COPYING\">" +
                     "GNU General Public License for more details.</P>" +
                     "You should have received a copy of the GNU General " +
                     "Public License along with this program; if not, write " +
                     "to the Free Software Foundation, Inc., 675 Mass Ave, " +
                     "Cambridge, MA 02139, USA.</P>" +
                     "<P>(See the file \"<A Href=\"COPYING\">COPYING</A>\" " +
                     "that is included with this distribution.)</P>" +
                     "<P>This product includes software developed by the " +
                     "Apache Software Foundation (<A Href=\"http://www.apache.org/\">" +
                     "http://www.apache.org/</A>).</P>";
  String footer    = "</FONT></BODY></HTML>";
  JLabel aboutIconLabel = new JLabel();
  JEditorPane commentsPane = new JEditorPane();
  Border border1;
  JScrollPane commentsScrollPane = new JScrollPane();

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a MainFrame_AboutBox.
   *
   * @param parent The parent frame.
   */
  public MainFrame_AboutBox(Frame parent)
  {
    super(parent);
    frame_ = parent;
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try
    {
      jbInit();
      mInit();
      pack();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  } //MainFrame_AboutBox

  //////////////////////////////////////////////////////////////////////////////
  // Private methods
  //////////////////////////////////////////////////////////////////////////////

  /* JBuilder Component initialization. */
  private void jbInit() throws Exception
  {
    setTitle("About");
    border1 = BorderFactory.createEmptyBorder(0,10,0,0);
    panel1.setLayout(borderLayout1);
    insetsPanel1.setLayout(flowLayout1);
    insetsPanel2.setLayout(flowLayout1);
    button1.setText("OK");
    button1.addActionListener(this);
    aboutIconLabel.setBorder(BorderFactory.createLoweredBevelBorder());
    aboutIconLabel.setMaximumSize(new Dimension(32767, 32767));
    aboutIconLabel.setPreferredSize(new Dimension(204, 154));
    aboutIconLabel.setIcon(aboutIcon);
    panel1.setBorder(BorderFactory.createEtchedBorder());
    panel1.setToolTipText("");
    commentsPane.setPreferredSize(new Dimension(300, 154));
    commentsPane.setVerifyInputWhenFocusTarget(false);
    commentsPane.setBackground(this.getBackground());
    commentsPane.setBorder(null);
    commentsPane.setMinimumSize(new Dimension(6, 10));
    commentsPane.setEditable(false);
    commentsPane.addHyperlinkListener(new javax.swing.event.HyperlinkListener()
    {
      public void hyperlinkUpdate(HyperlinkEvent e)
      {
        commentsPane_hyperlinkUpdate(e);
      }
    });
    commentsScrollPane.setBorder(BorderFactory.createLoweredBevelBorder());
    commentsScrollPane.setPreferredSize(new Dimension(304, 154));
    insetsPanel2.setPreferredSize(new Dimension(530, 160));
    flowLayout1.setHgap(10);
    panel1.add(insetsPanel2, BorderLayout.NORTH);
    insetsPanel2.add(aboutIconLabel, null);
    insetsPanel2.add(commentsScrollPane, null);
    commentsScrollPane.getViewport().add(commentsPane, null);
    insetsPanel1.add(button1, null);
    panel1.add(insetsPanel1, BorderLayout.SOUTH);
    getContentPane().add(panel1, null);
  } //jbInit

  /* Manual initialization. */
  private void mInit()
  {
    String base;

    // Create header
    try
    {
      base = new File(".").toURL().toString();
    }
    catch (MalformedURLException e)
    {
      base = new String();
      e.printStackTrace();
    }
    header = "<HTML><HEAD><BASE Href=\"" + base +
             "\"></A></HEAD><BODY><FONT Face=\"Dialog\" Size=\"2\">";
    // retrieve Look and Feels
    UIManager.LookAndFeelInfo[] lookandfeels;
    String lookfeels = "<P>Installed Swing Look and Feels: ";

    lookandfeels = UIManager.getInstalledLookAndFeels();
    for (int index = 0; index < lookandfeels.length; index++)
    {
      lookfeels += lookandfeels[index].getName() + ", ";
    }
    lookfeels = lookfeels.substring(0, lookfeels.length() - 2);
    lookfeels += ".</P>";
    // switch to HTML
    commentsPane.setContentType("text/html");
    commentsPane.setText(header + product + version + comments + copyright +
      lookfeels + footer);
    // scroll to top
    commentsPane.setCaretPosition(0);
  } //mInit

  /* Closes window. */
  void cancel()
  {
    dispose();
  } //cancel

  /* Handles hyperlink click. */
  void commentsPane_hyperlinkUpdate(HyperlinkEvent e)
  {
    if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
    {
      try
      {
        TextViewDialog viewDlg = new TextViewDialog(frame_, e.getDescription(),
          true);
        viewDlg.setPage(e.getURL());
        viewDlg.show();
      }
      catch (Exception exc)
      {
        commentsPane.setText(header + "Error: could not open " +
          exc.getMessage() + footer);
      }
    }
  } //commentsPane_hyperlinkUpdate

  //////////////////////////////////////////////////////////////////////////////
  // Protected methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Overridden event processing to link close event to button.
   *
   * @param e The window event.
   */
  protected void processWindowEvent(WindowEvent e)
  {
    if(e.getID() == WindowEvent.WINDOW_CLOSING)
    {
      cancel();
    }
    super.processWindowEvent(e);
  } //processWindowEvent

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Closes window.
   *
   * @param e The action event.
   */
  public void actionPerformed(ActionEvent e)
  {
    if(e.getSource() == button1)
    {
      cancel();
    }
  } //actionPerformed
} //class MainFrame_AboutBox
