/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.gui;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.util.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.BadLocationException;
import org.xml.sax.SAXException;
import EDU.auburn.VGJ.graph.*;
import EDU.auburn.VGJ.util.*;
import EDU.auburn.VGJ.algorithm.*;
import EDU.utwente.CoCompose.algorithm.*;
import EDU.utwente.CoCompose.lang.*;
import EDU.utwente.CoCompose.translate.*;
import EDU.utwente.CoCompose.util.*;
import EDU.utwente.CoCompose.xml.SAXWriter;

/**
 * Design Modeler window. Contains the actual design model and has all the
 * editing functionality.
 *
 * @author Dennis Wagelaar
 * @version 0.1.15, 20 October 2003
 */

public class ModelFrame extends JFrame implements ErrorReporter, Logger
{
  //////////////////////////////////////////////////////////////////////////////
  // Constants
  //////////////////////////////////////////////////////////////////////////////

  private static final String GENIMPL_PATH = "translate.";

  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  private double scale_ = 1.0;
  private RepositoryContainer repository_;
  private ModelCanvas modelCanvas_;
  private GraphAlgorithm layoutAlg_;
  private AngleControlPanel viewAnglePanel = new AngleControlPanel();
  private ViewPortPanel viewOffsetPanel = new ViewPortPanel();
  private DefaultStyledDocument log = new DefaultStyledDocument();
  private boolean fullFlatten_ = true;
  private int logLevel_ = 0;
  private String logIndent_ = new String();
  private DateFormat logFormatter_ = new SimpleDateFormat("HH:mm:ss ");
  private StringBuffer logBuffer_ = new StringBuffer(65535);
  private Date oldLogDate_ = new Date();
  private Logger logger_ = null;
  private ModelSpring modelSpring_ = new ModelSpring();
  private FlattenVisitorDepthFirst currentFlatten_ = null;
  private int currentFlattenCount_ = 1;
  ImageIcon appIcon = new ImageIcon("images/cocompose_icon.gif");
  ImageIcon closeIcon = new ImageIcon("images/door.gif");
  ImageIcon compositeIcon = new ImageIcon("images/composite.gif");
  ImageIcon compositePIcon = new ImageIcon("images/composite_p.gif");
  ImageIcon compositeUIcon = new ImageIcon("images/composite_u.gif");
  ImageIcon compositeRoleIcon = new ImageIcon("images/compositerole.gif");
  ImageIcon compositeRolePIcon = new ImageIcon("images/compositerole_p.gif");
  ImageIcon compositeRoleUIcon = new ImageIcon("images/compositerole_u.gif");
  ImageIcon conceptIcon = new ImageIcon("images/concept.gif");
  ImageIcon conceptPIcon = new ImageIcon("images/concept_p.gif");
  ImageIcon conceptUIcon = new ImageIcon("images/concept_u.gif");
  ImageIcon deleteIcon = new ImageIcon("images/delete.gif");
  ImageIcon deleteDIcon = new ImageIcon("images/delete_d.gif");
  ImageIcon determineIcon = new ImageIcon("images/concept_form.gif");
  ImageIcon editIcon = new ImageIcon("images/edit.gif");
  ImageIcon emptyIcon = new ImageIcon("images/empty.gif");
  ImageIcon fileIcon = new ImageIcon("images/new.gif");
  ImageIcon flattenIcon = new ImageIcon("images/flatten.gif");
  ImageIcon flattenOneIcon = new ImageIcon("images/flatten1.gif");
  ImageIcon imageIcon1 = new ImageIcon("images/flatten1.gif");
  ImageIcon flattenOneDIcon = new ImageIcon("images/flatten1_d.gif");
  ImageIcon generateCodeIcon = new ImageIcon("images/generate_code.gif");
  ImageIcon linkIcon = new ImageIcon("images/link.gif");
  ImageIcon linkPIcon = new ImageIcon("images/link_p.gif");
  ImageIcon linkUIcon = new ImageIcon("images/link_u.gif");
  ImageIcon modelIcon = new ImageIcon("images/model.gif");
  ImageIcon openIcon = new ImageIcon("images/open.gif");
  ImageIcon propertiesIcon = new ImageIcon("images/properties.gif");
  ImageIcon propertiesDIcon = new ImageIcon("images/properties_d.gif");
  ImageIcon pubConceptIcon = new ImageIcon("images/pconcept.gif");
  ImageIcon pubConceptPIcon = new ImageIcon("images/pconcept_p.gif");
  ImageIcon pubConceptUIcon = new ImageIcon("images/concept_u.gif");
  ImageIcon redoIcon = new ImageIcon("images/redo.gif");
  ImageIcon redoDIcon = new ImageIcon("images/redo_d.gif");
  ImageIcon reposIcon = new ImageIcon("images/repos.gif");
  ImageIcon reposPIcon = new ImageIcon("images/repos_p.gif");
  ImageIcon reposUIcon = new ImageIcon("images/repos_u.gif");
  ImageIcon saveIcon = new ImageIcon("images/save.gif");
  ImageIcon saveDIcon = new ImageIcon("images/save_d.gif");
  ImageIcon scatterIcon = new ImageIcon("images/scatter.gif");
  ImageIcon selectAllIcon = new ImageIcon("images/select_all.gif");
  ImageIcon selectIcon = new ImageIcon("images/select.gif");
  ImageIcon selectPIcon = new ImageIcon("images/select_p.gif");
  ImageIcon selectUIcon = new ImageIcon("images/select_u.gif");
  ImageIcon showPanelIcon = new ImageIcon("images/show_panel.gif");
  ImageIcon showTextIcon = new ImageIcon("images/show_text.gif");
  ImageIcon solutionRoleIcon = new ImageIcon("images/solutionrole.gif");
  ImageIcon solutionRolePIcon = new ImageIcon("images/solutionrole_p.gif");
  ImageIcon solutionRoleUIcon = new ImageIcon("images/solutionrole_u.gif");
  ImageIcon turnToConceptIcon = new ImageIcon("images/turn_to_concept.gif");
  ImageIcon turnToConceptDIcon = new ImageIcon("images/turn_to_concept_d.gif");
  ImageIcon turnToCompositeIcon = new ImageIcon("images/turn_to_composite.gif");
  ImageIcon turnToCompositeDIcon = new ImageIcon("images/turn_to_composite_d.gif");
  ImageIcon turnToRoleIcon = new ImageIcon("images/turn_to_role.gif");
  ImageIcon turnToRoleDIcon = new ImageIcon("images/turn_to_role_d.gif");
  ImageIcon turnToReposIcon = new ImageIcon("images/turn_to_repos.gif");
  ImageIcon turnToReposDIcon = new ImageIcon("images/turn_to_repos_d.gif");
  ImageIcon undoIcon = new ImageIcon("images/undo.gif");
  ImageIcon undoDIcon = new ImageIcon("images/undo_d.gif");
  ImageIcon viewSourceIcon = new ImageIcon("images/view_source.gif");
  ImageIcon zoomIcon = new ImageIcon("images/zoom.gif");
  BorderLayout borderLayout1 = new BorderLayout();
  CompoundBorder titledBorder1;
  CompoundBorder titledBorder2;
  CompoundBorder titledBorder3;
  CompoundBorder titledBorder4;
  CompoundBorder titledBorder6;
  CompoundBorder titledBorder5;
  CompoundBorder titledBorder7;
  JMenuBar menuBar1 = new JMenuBar();
  JMenu menuFile = new JMenu();
  JMenuItem menuSave = new JMenuItem();
  JMenuItem menuClose = new JMenuItem();
  JMenu menuView = new JMenu();
  JMenu menuEdit = new JMenu();
  JMenuItem menuSelectAll = new JMenuItem();
  JMenuItem menuDeselectAll = new JMenuItem();
  JMenuItem menuDelete = new JMenuItem();
  JPanel controls1 = new JPanel();
  JRadioButton createLinkRadio = new JRadioButton();
  JRadioButton createCompositeRadio = new JRadioButton();
  JPanel mouseActionPanel = new JPanel();
  JRadioButton selectRadio = new JRadioButton();
  GridLayout gridLayout7 = new GridLayout();
  JRadioButton createConceptRadio = new JRadioButton();
  JRadioButton createRoleRadio = new JRadioButton();
  JPanel controls3 = new JPanel();
  BorderLayout borderLayout3 = new BorderLayout();
  JPanel controls4 = new JPanel();
  JButton halfScaleBtn = new JButton();
  JButton normalScaleBtn = new JButton();
  JPanel scalePanel = new JPanel();
  JButton doubleScaleBtn = new JButton();
  BorderLayout borderLayout4 = new BorderLayout();
  JPanel controls5 = new JPanel();
  BorderLayout borderLayout5 = new BorderLayout();
  BorderLayout borderLayout6 = new BorderLayout();
  JMenuItem menuProperties = new JMenuItem();
  JPanel controls2 = new JPanel();
  BorderLayout borderLayout8 = new BorderLayout();
  JMenuItem menuRepos = new JMenuItem();
  JRadioButton createFeatReposRadio = new JRadioButton();
  JMenuItem menuTurnToComposite = new JMenuItem();
  JMenuItem menuTurnToConcept = new JMenuItem();
  JMenuItem menuTurnToRepos = new JMenuItem();
  JRadioButton createSolRoleRadio = new JRadioButton();
  JMenuItem menuTurnToSolutionRole = new JMenuItem();
  JMenu menuTranslate = new JMenu();
  JMenuItem menuFlatten = new JMenuItem();
  JMenuItem menuGenerate = new JMenuItem();
  JPanel controls6 = new JPanel();
  BorderLayout borderLayout9 = new BorderLayout();
  JPanel targetLangPanel = new JPanel();
  JTextField targetLanguageEdit = new JTextField();
  FlowLayout flowLayout3 = new FlowLayout();
  JButton editButton = new JButton();
  FlowLayout flowLayout4 = new FlowLayout();
  JPanel controls = new JPanel();
  BorderLayout borderLayout10 = new BorderLayout();
  JPanel menuPanel = new JPanel();
  BorderLayout borderLayout11 = new BorderLayout();
  JButton saveBtn = new JButton();
  GridLayout gridLayout1 = new GridLayout();
  JButton reposBtn = new JButton();
  JMenuItem menuViewXML = new JMenuItem();
  JCheckBoxMenuItem menuShowAngle = new JCheckBoxMenuItem();
  JCheckBoxMenuItem menuShowJMenu = new JCheckBoxMenuItem();
  JCheckBoxMenuItem menuShowMouse = new JCheckBoxMenuItem();
  JCheckBoxMenuItem menuShowOffset = new JCheckBoxMenuItem();
  JCheckBoxMenuItem menuShowScale = new JCheckBoxMenuItem();
  JCheckBoxMenuItem menuShowLang = new JCheckBoxMenuItem();
  TitledBorder titledBorder8;
  JPopupMenu popupMenu1 = new JPopupMenu();
  JMenuItem popupDelete = new JMenuItem();
  JMenuItem popupProperties = new JMenuItem();
  JMenuItem popupTurnToComposite = new JMenuItem();
  JMenuItem popupTurnToConcept = new JMenuItem();
  JMenuItem popupTurnToSolutionRole = new JMenuItem();
  JMenuItem popupTurnToRepos = new JMenuItem();
  JMenuItem menuEPS = new JMenuItem();
  JMenuItem menuFlattenOne = new JMenuItem();
  JCheckBoxMenuItem menuShowLog = new JCheckBoxMenuItem();
  TitledBorder titledBorder9;
  JPanel rightPanel = new JPanel();
  BorderLayout borderLayout7 = new BorderLayout();
  ModelPanel viewingPanel = new ModelPanel();
  JPanel logPanel = new JPanel();
  JScrollPane logScrollPane = new JScrollPane();
  BorderLayout borderLayout2 = new BorderLayout();
  JTextPane logPane = new JTextPane(log);
  JMenuItem menuDetermine = new JMenuItem();
  JMenu modelMenu = new JMenu();
  JMenuItem modelPropertiesMenuItem = new JMenuItem();
  JMenuItem menuPosition = new JMenuItem();
  JMenuItem menuResetForms = new JMenuItem();
  JCheckBoxMenuItem menuLogToggle = new JCheckBoxMenuItem();
  JCheckBoxMenuItem menuScatter = new JCheckBoxMenuItem();
  JMenuItem menuClearLog = new JMenuItem();
  JPopupMenu logPopupMenu = new JPopupMenu();
  JMenuItem popupHideLog = new JMenuItem();
  JMenuItem popupClearLog = new JMenuItem();
  JRadioButton createPublishedRadio = new JRadioButton();
  JMenuItem menuTranslateAll = new JMenuItem();
  JMenuItem menuUndo = new JMenuItem();
  JMenuItem menuRedo = new JMenuItem();
  JMenuItem menuUpdateComposites = new JMenuItem();
  JMenuItem menuFlattenDepthFirst = new JMenuItem();
  JMenuItem menuNextFlattenOption = new JMenuItem();
  JMenu menuFlattenBreadthFirst = new JMenu();
  JMenuItem menuSearch = new JMenuItem();
  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a ModelFrame given a particular model.
   *
   * @param model_in The model object to use.
   */
  public ModelFrame(Model model_in)
  {
    super();
    modelCanvas_ = new ModelCanvas(model_in, this);
    viewingPanel = new ModelPanel(modelCanvas_);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try
    {
      jbInit();
      mInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  } //ModelFrame(Model model_in)

  /**
   * Constructs a ModelFrame.
   * This constructor is only for the JBuilder Designer, do not use it!
   */
  public ModelFrame()
  {
    this(new Model());
  } //ModelFrame()

  //////////////////////////////////////////////////////////////////////////////
  // Private methods
  //////////////////////////////////////////////////////////////////////////////

  /* JBuilder Component initialization. */
  private void jbInit() throws Exception
  {
    menuFlattenOne.setIcon(imageIcon1);
    menuFlattenOne.setText("Flatten one level");
    menuFlattenOne.setAccelerator(javax.swing.KeyStroke.getKeyStroke(117, java.awt.event.KeyEvent.ALT_MASK + java.awt.event.KeyEvent.SHIFT_MASK, false));
    menuFlattenOne.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuFlattenOne_actionPerformed(e);
      }
    });
    menuFlatten.setIcon(flattenIcon);
    menuFlatten.setText("Flatten all");
    menuFlatten.setAccelerator(javax.swing.KeyStroke.getKeyStroke(117, java.awt.event.KeyEvent.ALT_MASK, false));
    menuFlatten.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuFlatten_actionPerformed(e);
      }
    });
    titledBorder1 = BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new java.awt.Color(148, 145, 140)),"Mouse action"),BorderFactory.createEmptyBorder(0,1,1,1));
    titledBorder2 = BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new java.awt.Color(148, 145, 140)),"Viewing offset"),BorderFactory.createEmptyBorder(0,1,1,1));
    titledBorder3 = BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new java.awt.Color(148, 145, 140)),"Scale: 1.0"),BorderFactory.createEmptyBorder(0,1,1,1));
    titledBorder4 = BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(148, 145, 140)),"Viewing angles"),BorderFactory.createEmptyBorder(0,1,1,1));
    titledBorder5 = BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new java.awt.Color(148, 145, 140)),"Model view"),BorderFactory.createEmptyBorder(0,1,1,1));
    titledBorder6 = BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new java.awt.Color(148, 145, 140)),"Menu"),BorderFactory.createEmptyBorder(0,1,1,1));
    titledBorder7 = BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new java.awt.Color(148, 145, 140)),"Target language"),BorderFactory.createEmptyBorder(0,1,1,1));
    titledBorder8 = new TitledBorder("");
    titledBorder9 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(148, 145, 140)),"Log");
    this.getContentPane().setLayout(borderLayout1);
    menuFile.setIcon(fileIcon);
    menuFile.setText("File");
    menuSave.setIcon(saveIcon);
    menuSave.setText("Save model");
    menuSave.setAccelerator(javax.swing.KeyStroke.getKeyStroke(83, java.awt.event.KeyEvent.CTRL_MASK, false));
    menuSave.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuSave_actionPerformed(e);
      }
    });
    menuClose.setIcon(closeIcon);
    menuClose.setText("Close");
    menuClose.setAccelerator(javax.swing.KeyStroke.getKeyStroke(115, java.awt.event.KeyEvent.ALT_MASK, false));
    menuClose.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuClose_actionPerformed(e);
      }
    });
    menuView.setIcon(zoomIcon);
    menuView.setText("View");
    menuEdit.setIcon(editIcon);
    menuEdit.setText("Edit");
    menuSelectAll.setIcon(selectAllIcon);
    menuSelectAll.setText("Select all");
    menuSelectAll.setAccelerator(javax.swing.KeyStroke.getKeyStroke(65, java.awt.event.KeyEvent.CTRL_MASK, false));
    menuSelectAll.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuSelectAll_actionPerformed(e);
      }
    });
    menuDeselectAll.setDisabledIcon(selectUIcon);
    menuDeselectAll.setIcon(selectIcon);
    menuDeselectAll.setText("Deselect all");
    menuDeselectAll.setAccelerator(javax.swing.KeyStroke.getKeyStroke(68, java.awt.event.KeyEvent.CTRL_MASK, false));
    menuDeselectAll.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuDeselectAll_actionPerformed(e);
      }
    });
    menuDelete.setDisabledIcon(deleteDIcon);
    menuDelete.setIcon(deleteIcon);
    menuDelete.setText("Delete");
    menuDelete.setAccelerator(javax.swing.KeyStroke.getKeyStroke(127, 0, false));
    menuDelete.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuDelete_actionPerformed(e);
      }
    });
    controls.setLayout(borderLayout3);
    createLinkRadio.addChangeListener(new javax.swing.event.ChangeListener()
    {

      public void stateChanged(ChangeEvent e)
      {
        createLinkRadio_stateChanged(e);
      }
    });
    createLinkRadio.setIcon(linkUIcon);
    createLinkRadio.setPressedIcon(linkPIcon);
    createLinkRadio.setSelectedIcon(linkIcon);
    createLinkRadio.setText("Create link");
    createCompositeRadio.addChangeListener(new javax.swing.event.ChangeListener()
    {

      public void stateChanged(ChangeEvent e)
      {
        createCompositeRadio_stateChanged(e);
      }
    });
    createCompositeRadio.setIcon(compositeUIcon);
    createCompositeRadio.setPressedIcon(compositePIcon);
    createCompositeRadio.setSelectedIcon(compositeIcon);
    createCompositeRadio.setText("Create composite");
    mouseActionPanel.setBorder(titledBorder1);
    mouseActionPanel.setLayout(gridLayout7);
    selectRadio.setIcon(selectUIcon);
    selectRadio.setPressedIcon(selectPIcon);
    selectRadio.setSelected(true);
    selectRadio.setSelectedIcon(selectIcon);
    selectRadio.setText("Select");
    selectRadio.addChangeListener(new javax.swing.event.ChangeListener()
    {

      public void stateChanged(ChangeEvent e)
      {
        selectRadio_stateChanged(e);
      }
    });
    gridLayout7.setRows(8);
    gridLayout7.setColumns(1);
    createConceptRadio.setIcon(conceptUIcon);
    createConceptRadio.setPressedIcon(conceptPIcon);
    createConceptRadio.setSelectedIcon(conceptIcon);
    createConceptRadio.setText("Create concept");
    createConceptRadio.addChangeListener(new javax.swing.event.ChangeListener()
    {

      public void stateChanged(ChangeEvent e)
      {
        createConceptRadio_stateChanged(e);
      }
    });
    createRoleRadio.setIcon(compositeRoleUIcon);
    createRoleRadio.setPressedIcon(compositeRolePIcon);
    createRoleRadio.setSelectedIcon(compositeRoleIcon);
    createRoleRadio.setText("Create composite role");
    createRoleRadio.addChangeListener(new javax.swing.event.ChangeListener()
    {

      public void stateChanged(ChangeEvent e)
      {
        createRoleRadio_stateChanged(e);
      }
    });
    controls2.setLayout(borderLayout4);
    viewOffsetPanel.setBorder(titledBorder2);
    viewOffsetPanel.addChangeListener(new javax.swing.event.ChangeListener()
    {
      public void stateChanged(ChangeEvent e)
      {
        viewOffsetPanel_stateChanged(e);
      }
    });
    controls3.setLayout(borderLayout6);
    halfScaleBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        halfScaleBtn_actionPerformed(e);
      }
    });
    halfScaleBtn.setText("/ 2");
    halfScaleBtn.setIcon(zoomIcon);
    halfScaleBtn.setMargin(new Insets(2, 8, 2, 8));
    normalScaleBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        normalScaleBtn_actionPerformed(e);
      }
    });
    normalScaleBtn.setText("= 1");
    normalScaleBtn.setIcon(zoomIcon);
    normalScaleBtn.setMargin(new Insets(2, 8, 2, 8));
    scalePanel.setLayout(flowLayout4);
    scalePanel.setBorder(titledBorder3);
    doubleScaleBtn.setIcon(zoomIcon);
    doubleScaleBtn.setMargin(new Insets(2, 8, 2, 8));
    doubleScaleBtn.setText("* 2");
    doubleScaleBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        doubleScaleBtn_actionPerformed(e);
      }
    });
    viewAnglePanel.setBorder(titledBorder4);
    viewAnglePanel.addChangeListener(new javax.swing.event.ChangeListener()
    {
      public void stateChanged(ChangeEvent e)
      {
        viewAnglePanel_stateChanged(e);
      }
    });
    controls4.setLayout(borderLayout5);
    menuProperties.setDisabledIcon(propertiesDIcon);
    menuProperties.setIcon(propertiesIcon);
    menuProperties.setText("Properties");
    menuProperties.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuProperties_actionPerformed(e);
      }
    });
    controls1.setLayout(borderLayout8);
    menuRepos.setDisabledIcon(saveDIcon);
    menuRepos.setIcon(saveIcon);
    menuRepos.setText("Save composite in repository");
    menuRepos.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuRepos_actionPerformed(e);
      }
    });
    createFeatReposRadio.setIcon(reposUIcon);
    createFeatReposRadio.setPressedIcon(reposPIcon);
    createFeatReposRadio.setSelectedIcon(reposIcon);
    createFeatReposRadio.setText("Create repository composite");
    createFeatReposRadio.addChangeListener(new javax.swing.event.ChangeListener()
    {

      public void stateChanged(ChangeEvent e)
      {
        createFeatReposRadio_stateChanged(e);
      }
    });
    menuTurnToComposite.setDisabledIcon(turnToCompositeDIcon);
    menuTurnToComposite.setIcon(turnToCompositeIcon);
    menuTurnToComposite.setText("Turn into composite");
    menuTurnToComposite.setAccelerator(javax.swing.KeyStroke.getKeyStroke(70, java.awt.event.KeyEvent.CTRL_MASK, false));
    menuTurnToComposite.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuTurnToComposite_actionPerformed(e);
      }
    });
    menuTurnToConcept.setDisabledIcon(turnToConceptDIcon);
    menuTurnToConcept.setIcon(turnToConceptIcon);
    menuTurnToConcept.setText("Turn into concept");
    menuTurnToConcept.setAccelerator(javax.swing.KeyStroke.getKeyStroke(67, java.awt.event.KeyEvent.CTRL_MASK, false));
    menuTurnToConcept.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuTurnToConcept_actionPerformed(e);
      }
    });
    menuTurnToRepos.setDisabledIcon(turnToReposDIcon);
    menuTurnToRepos.setIcon(turnToReposIcon);
    menuTurnToRepos.setText("Turn into composite from repository");
    menuTurnToRepos.setAccelerator(javax.swing.KeyStroke.getKeyStroke(84, java.awt.event.KeyEvent.CTRL_MASK, false));
    menuTurnToRepos.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuTurnToRepos_actionPerformed(e);
      }
    });
    createSolRoleRadio.setActionCommand("Create solution role");
    createSolRoleRadio.setIcon(solutionRoleUIcon);
    createSolRoleRadio.setPressedIcon(solutionRolePIcon);
    createSolRoleRadio.setSelectedIcon(solutionRoleIcon);
    createSolRoleRadio.setText("Create solution role");
    createSolRoleRadio.addChangeListener(new javax.swing.event.ChangeListener()
    {

      public void stateChanged(ChangeEvent e)
      {
        createSolRoleRadio_stateChanged(e);
      }
    });
    menuTurnToSolutionRole.setDisabledIcon(turnToRoleDIcon);
    menuTurnToSolutionRole.setIcon(turnToRoleIcon);
    menuTurnToSolutionRole.setText("Turn into solution role");
    menuTurnToSolutionRole.setAccelerator(javax.swing.KeyStroke.getKeyStroke(82, java.awt.event.KeyEvent.CTRL_MASK, false));
    menuTurnToSolutionRole.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuTurnToSolutionRole_actionPerformed(e);
      }
    });
    menuTranslate.setIcon(generateCodeIcon);
    menuTranslate.setText("Translate");
    menuGenerate.setIcon(generateCodeIcon);
    menuGenerate.setText("Generate implementation");
    menuGenerate.setAccelerator(javax.swing.KeyStroke.getKeyStroke(119, java.awt.event.KeyEvent.CTRL_MASK, false));
    menuGenerate.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuGenerate_actionPerformed(e);
      }
    });
    controls5.setLayout(borderLayout9);
    targetLangPanel.setBorder(titledBorder7);
    targetLangPanel.setLayout(flowLayout3);
    targetLanguageEdit.setBackground(getBackground());
    targetLanguageEdit.setPreferredSize(new Dimension(120, 21));
    targetLanguageEdit.setEditable(false);
    editButton.setMargin(new Insets(2, 8, 2, 8));
    editButton.setText("Edit");
    editButton.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        editButton_actionPerformed(e);
      }
    });
    flowLayout4.setHgap(0);
    controls.setLayout(borderLayout10);
    menuPanel.setBorder(titledBorder6);
    menuPanel.setLayout(gridLayout1);
    controls6.setLayout(borderLayout11);
    saveBtn.setHorizontalAlignment(SwingConstants.LEFT);
    saveBtn.setIcon(saveIcon);
    saveBtn.setText("Save model");
    saveBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuSave_actionPerformed(e);
      }
    });
    gridLayout1.setColumns(1);
    gridLayout1.setRows(2);
    reposBtn.setDisabledIcon(saveDIcon);
    reposBtn.setHorizontalAlignment(SwingConstants.LEFT);
    reposBtn.setIcon(saveIcon);
    reposBtn.setText("Save composite");
    reposBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuRepos_actionPerformed(e);
      }
    });
    menuViewXML.setText("View XML source");
    menuViewXML.setIcon(viewSourceIcon);
    menuViewXML.setAccelerator(javax.swing.KeyStroke.getKeyStroke(83, java.awt.event.KeyEvent.ALT_MASK, false));
    menuViewXML.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuViewXML_actionPerformed(e);
      }
    });
    menuShowAngle.setIcon(showPanelIcon);
    menuShowAngle.setText("Show viewing angle panel");
    menuShowAngle.addItemListener(new java.awt.event.ItemListener()
    {
      public void itemStateChanged(ItemEvent e)
      {
        menuShowAngle_itemStateChanged(e);
      }
    });
    menuShowJMenu.setIcon(showPanelIcon);
    menuShowJMenu.setText("Show menu panel");
    menuShowJMenu.setState(true);
    menuShowJMenu.addItemListener(new java.awt.event.ItemListener()
    {
      public void itemStateChanged(ItemEvent e)
      {
        menuShowJMenu_itemStateChanged(e);
      }
    });
    menuShowMouse.setIcon(showPanelIcon);
    menuShowMouse.setText("Show mouse action panel");
    menuShowMouse.setState(true);
    menuShowMouse.addItemListener(new java.awt.event.ItemListener()
    {
      public void itemStateChanged(ItemEvent e)
      {
        menuShowMouse_itemStateChanged(e);
      }
    });
    menuShowOffset.setIcon(showPanelIcon);
    menuShowOffset.setText("Show offset panel");
    menuShowOffset.setState(true);
    menuShowOffset.addItemListener(new java.awt.event.ItemListener()
    {
      public void itemStateChanged(ItemEvent e)
      {
        menuShowOffset_itemStateChanged(e);
      }
    });
    menuShowScale.setIcon(showPanelIcon);
    menuShowScale.setText("Show scale panel");
    menuShowScale.setState(true);
    menuShowScale.addItemListener(new java.awt.event.ItemListener()
    {
      public void itemStateChanged(ItemEvent e)
      {
        menuShowScale_itemStateChanged(e);
      }
    });
    menuShowLang.setIcon(showPanelIcon);
    menuShowLang.setText("Show target language panel");
    menuShowLang.setState(true);
    menuShowLang.addItemListener(new java.awt.event.ItemListener()
    {
      public void itemStateChanged(ItemEvent e)
      {
        menuShowLang_itemStateChanged(e);
      }
    });
    popupDelete.setIcon(deleteIcon);
    popupDelete.setText("Delete");
    popupDelete.setAccelerator(javax.swing.KeyStroke.getKeyStroke(127, 0, false));
    popupDelete.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuDelete_actionPerformed(e);
      }
    });
    popupProperties.setIcon(propertiesIcon);
    popupProperties.setText("Edit properties");
    popupProperties.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuProperties_actionPerformed(e);
      }
    });
    popupTurnToComposite.setIcon(turnToCompositeIcon);
    popupTurnToComposite.setText("Turn into composite");
    popupTurnToComposite.setAccelerator(javax.swing.KeyStroke.getKeyStroke(70, java.awt.event.KeyEvent.CTRL_MASK, false));
    popupTurnToComposite.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuTurnToComposite_actionPerformed(e);
      }
    });
    popupTurnToConcept.setIcon(turnToConceptIcon);
    popupTurnToConcept.setText("Turn into concept");
    popupTurnToConcept.setAccelerator(javax.swing.KeyStroke.getKeyStroke(67, java.awt.event.KeyEvent.CTRL_MASK, false));
    popupTurnToConcept.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuTurnToConcept_actionPerformed(e);
      }
    });
    popupTurnToSolutionRole.setActionCommand("Turn into solution role");
    popupTurnToSolutionRole.setIcon(turnToRoleIcon);
    popupTurnToSolutionRole.setText("Turn into solution role");
    popupTurnToSolutionRole.setAccelerator(javax.swing.KeyStroke.getKeyStroke(82, java.awt.event.KeyEvent.CTRL_MASK, false));
    popupTurnToSolutionRole.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuTurnToSolutionRole_actionPerformed(e);
      }
    });
    popupTurnToRepos.setIcon(turnToReposIcon);
    popupTurnToRepos.setMnemonic('0');
    popupTurnToRepos.setText("Turn into composite from repository");
    popupTurnToRepos.setAccelerator(javax.swing.KeyStroke.getKeyStroke(84, java.awt.event.KeyEvent.CTRL_MASK, false));
    popupTurnToRepos.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuTurnToRepos_actionPerformed(e);
      }
    });
    modelCanvas_.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mouseReleased(MouseEvent e)
      {
        modelCanvas__mouseReleased(e);
      }
    });
    menuEPS.setIcon(saveIcon);
    menuEPS.setText("Save model as EPS");
    menuEPS.setAccelerator(javax.swing.KeyStroke.getKeyStroke(69, java.awt.event.KeyEvent.CTRL_MASK, false));
    menuEPS.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuEPS_actionPerformed(e);
      }
    });
    menuShowLog.setIcon(showTextIcon);
    menuShowLog.setText("Show logging pane");
    menuShowLog.setAccelerator(javax.swing.KeyStroke.getKeyStroke(76, java.awt.event.KeyEvent.ALT_MASK, false));
    menuShowLog.addItemListener(new java.awt.event.ItemListener()
    {
      public void itemStateChanged(ItemEvent e)
      {
        menuShowLog_itemStateChanged(e);
      }
    });
    rightPanel.setLayout(borderLayout7);
    viewingPanel.setBorder(titledBorder5);
    viewingPanel.addChangeListener(new javax.swing.event.ChangeListener()
    {
      public void stateChanged(ChangeEvent e)
      {
        viewingPanel_stateChanged(e);
      }
    });
    logPanel.setBorder(titledBorder9);
    logPanel.setLayout(borderLayout2);
    logScrollPane.setBorder(BorderFactory.createLoweredBevelBorder());
    logScrollPane.setPreferredSize(new Dimension(10, 96));
    logPane.setPreferredSize(new Dimension(6, 92));
    logPane.setEditable(false);
    logPane.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mouseReleased(MouseEvent e)
      {
        logPane_mouseReleased(e);
      }
    });
    menuDetermine.setIcon(determineIcon);
    menuDetermine.setText("Determine concept forms");
    menuDetermine.setAccelerator(javax.swing.KeyStroke.getKeyStroke(118, java.awt.event.KeyEvent.CTRL_MASK, false));
    menuDetermine.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuDetermine_actionPerformed(e);
      }
    });
    modelMenu.setIcon(modelIcon);
    modelMenu.setText("Model");
    modelPropertiesMenuItem.setIcon(propertiesIcon);
    modelPropertiesMenuItem.setText("Model properties");
    modelPropertiesMenuItem.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        modelPropertiesMenuItem_actionPerformed(e);
      }
    });
    menuPosition.setIcon(openIcon);
    menuPosition.setText("Load positions");
    menuPosition.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuPosition_actionPerformed(e);
      }
    });
    menuResetForms.setIcon(conceptIcon);
    menuResetForms.setText("Reset concept forms");
    menuResetForms.setAccelerator(javax.swing.KeyStroke.getKeyStroke(118, java.awt.event.KeyEvent.CTRL_MASK + java.awt.event.KeyEvent.SHIFT_MASK, false));
    menuResetForms.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuResetForms_actionPerformed(e);
      }
    });
    menuLogToggle.setIcon(emptyIcon);
    menuLogToggle.setText("Toggle logging");
    menuLogToggle.setState(true);
    menuLogToggle.addItemListener(new java.awt.event.ItemListener()
    {
      public void itemStateChanged(ItemEvent e)
      {
        menuLogToggle_itemStateChanged(e);
      }
    });
    menuScatter.setIcon(scatterIcon);
    menuScatter.setText("Toggle scatter");
    menuScatter.addItemListener(new java.awt.event.ItemListener()
    {
      public void itemStateChanged(ItemEvent e)
      {
        menuScatter_itemStateChanged(e);
      }
    });
    menuClearLog.setIcon(deleteIcon);
    menuClearLog.setText("Clear log");
    menuClearLog.setAccelerator(javax.swing.KeyStroke.getKeyStroke(67, java.awt.event.KeyEvent.ALT_MASK, false));
    menuClearLog.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuClearLog_actionPerformed(e);
      }
    });
    popupHideLog.setIcon(showTextIcon);
    popupHideLog.setText("Hide");
    popupHideLog.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        popupHideLog_actionPerformed(e);
      }
    });
    popupClearLog.setIcon(deleteIcon);
    popupClearLog.setText("Clear");
    popupClearLog.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuClearLog_actionPerformed(e);
      }
    });
    this.addWindowListener(new java.awt.event.WindowAdapter()
    {
      public void windowClosed(WindowEvent e)
      {
        this_windowClosed(e);
      }
    });
    createPublishedRadio.setIcon(pubConceptUIcon);
    createPublishedRadio.setPressedIcon(pubConceptPIcon);
    createPublishedRadio.setSelectedIcon(pubConceptIcon);
    createPublishedRadio.setText("Create published concept");
    createPublishedRadio.addChangeListener(new javax.swing.event.ChangeListener()
    {
      public void stateChanged(ChangeEvent e)
      {
        createPublishedRadio_stateChanged(e);
      }
    });
    menuTranslateAll.setIcon(generateCodeIcon);
    menuTranslateAll.setText("Apply all");
    menuTranslateAll.setAccelerator(javax.swing.KeyStroke.getKeyStroke(120, java.awt.event.KeyEvent.CTRL_MASK, false));
    menuTranslateAll.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuTranslateAll_actionPerformed(e);
      }
    });
    menuUndo.setDisabledIcon(undoDIcon);
    menuUndo.setIcon(undoIcon);
    menuUndo.setText("Undo");
    menuUndo.setAccelerator(javax.swing.KeyStroke.getKeyStroke(90, java.awt.event.KeyEvent.CTRL_MASK, false));
    menuUndo.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuUndo_actionPerformed(e);
      }
    });
    menuRedo.setDisabledIcon(redoDIcon);
    menuRedo.setIcon(redoIcon);
    menuRedo.setText("Redo");
    menuRedo.setAccelerator(javax.swing.KeyStroke.getKeyStroke(90, java.awt.event.KeyEvent.CTRL_MASK | java.awt.event.KeyEvent.SHIFT_MASK, false));
    menuRedo.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuRedo_actionPerformed(e);
      }
    });
    menuUpdateComposites.setDisabledIcon(turnToReposDIcon);
    menuUpdateComposites.setIcon(turnToReposIcon);
    menuUpdateComposites.setText("Update composites from repository");
    menuUpdateComposites.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuUpdateComposites_actionPerformed(e);
      }
    });
    this.getContentPane().add(controls, BorderLayout.WEST);
    controls.add(controls1, BorderLayout.CENTER);
    controls1.add(menuPanel, BorderLayout.NORTH);
    menuPanel.add(saveBtn, null);
    menuPanel.add(reposBtn, null);
    controls1.add(controls2, BorderLayout.CENTER);
    controls2.add(mouseActionPanel, BorderLayout.NORTH);
    mouseActionPanel.add(selectRadio, null);
    mouseActionPanel.add(createConceptRadio, null);
    mouseActionPanel.add(createSolRoleRadio, null);
    mouseActionPanel.add(createCompositeRadio, null);
    mouseActionPanel.add(createRoleRadio, null);
    mouseActionPanel.add(createPublishedRadio, null);
    mouseActionPanel.add(createFeatReposRadio, null);
    mouseActionPanel.add(createLinkRadio, null);
    controls2.add(controls3, BorderLayout.CENTER);
    controls3.add(viewOffsetPanel, BorderLayout.NORTH);
    controls3.add(controls4, BorderLayout.CENTER);
    controls4.add(scalePanel, BorderLayout.NORTH);
    scalePanel.add(halfScaleBtn, null);
    scalePanel.add(normalScaleBtn, null);
    scalePanel.add(doubleScaleBtn, null);
    controls4.add(controls5, BorderLayout.CENTER);
    controls5.add(viewAnglePanel, BorderLayout.NORTH);
    controls5.add(controls6, BorderLayout.CENTER);
    controls6.add(targetLangPanel, BorderLayout.NORTH);
    targetLangPanel.add(targetLanguageEdit, null);
    targetLangPanel.add(editButton, null);
    this.getContentPane().add(rightPanel,  BorderLayout.CENTER);
    menuBar1.add(menuFile);
    menuBar1.add(menuEdit);
    menuBar1.add(menuView);
    menuBar1.add(menuTranslate);
    menuBar1.add(modelMenu);
    menuFile.add(menuSave);
    menuFile.add(menuEPS);
    menuFile.addSeparator();
    menuFile.add(menuRepos);
    menuFile.addSeparator();
    menuFile.add(menuClose);
    menuView.add(menuViewXML);
    menuView.addSeparator();
    menuView.add(menuShowJMenu);
    menuView.add(menuShowMouse);
    menuView.add(menuShowOffset);
    menuView.add(menuShowScale);
    menuView.add(menuShowAngle);
    menuView.add(menuShowLang);
    menuView.addSeparator();
    menuView.add(menuShowLog);
    menuView.add(menuLogToggle);
    menuView.add(menuClearLog);
    menuView.addSeparator();
    menuView.add(menuScatter);
    menuView.add(menuPosition);
    menuView.addSeparator();
    menuView.add(menuSearch);
    menuEdit.add(menuUndo);
    menuEdit.add(menuRedo);
    menuEdit.addSeparator();
    menuEdit.add(menuDelete);
    menuEdit.addSeparator();
    menuEdit.add(menuSelectAll);
    menuEdit.add(menuDeselectAll);
    menuEdit.addSeparator();
    menuEdit.add(menuTurnToComposite);
    menuEdit.add(menuTurnToConcept);
    menuEdit.add(menuTurnToSolutionRole);
    menuEdit.add(menuTurnToRepos);
    menuEdit.add(menuUpdateComposites);
    menuEdit.addSeparator();
    menuEdit.add(menuProperties);
    menuTranslate.add(menuFlattenDepthFirst);
    menuTranslate.add(menuNextFlattenOption);
    menuTranslate.add(menuFlattenBreadthFirst);
    menuTranslate.addSeparator();
    menuTranslate.add(menuDetermine);
    menuTranslate.add(menuResetForms);
    menuTranslate.addSeparator();
    menuTranslate.add(menuGenerate);
    menuTranslate.addSeparator();
    menuTranslate.add(menuTranslateAll);
    popupMenu1.add(popupDelete);
    popupMenu1.addSeparator();
    popupMenu1.add(popupTurnToComposite);
    popupMenu1.add(popupTurnToConcept);
    popupMenu1.add(popupTurnToSolutionRole);
    popupMenu1.add(popupTurnToRepos);
    popupMenu1.addSeparator();
    popupMenu1.add(popupProperties);
    rightPanel.add(viewingPanel,  BorderLayout.CENTER);
    rightPanel.add(logPanel,  BorderLayout.SOUTH);
    logPanel.add(logScrollPane, BorderLayout.CENTER);
    logScrollPane.getViewport().add(logPane, null);
    modelMenu.add(modelPropertiesMenuItem);
    logPopupMenu.add(popupHideLog);
    logPopupMenu.add(popupClearLog);
    setJMenuBar(menuBar1);
    menuFlattenDepthFirst.setText("Flatten depth-first");
    menuFlattenDepthFirst.setIcon(flattenOneIcon);
    menuFlattenDepthFirst.setAccelerator(javax.swing.KeyStroke.getKeyStroke(117, java.awt.event.KeyEvent.CTRL_MASK, false));
    menuNextFlattenOption.setText("Next flatten option");
    menuNextFlattenOption.setIcon(flattenOneIcon);
    menuNextFlattenOption.setDisabledIcon(flattenOneDIcon);
    menuNextFlattenOption.setAccelerator(javax.swing.KeyStroke.getKeyStroke(117, java.awt.event.KeyEvent.CTRL_MASK + java.awt.event.KeyEvent.SHIFT_MASK, false));
    menuNextFlattenOption.setEnabled(false);
    menuFlattenDepthFirst.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){menuFlattenDepthFirstActionPerformed(e);}});
    menuNextFlattenOption.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){menuNextFlattenOptionActionPerformed(e);}});
    menuFlattenBreadthFirst.setText("Flatten breadth-first");
    menuFlattenBreadthFirst.setIcon(flattenIcon);
    menuFlattenBreadthFirst.add(menuFlatten);
    menuFlattenBreadthFirst.add(menuFlattenOne);
    menuSearch.setText("Search...");
    menuSearch.setIcon(viewSourceIcon);
    menuSearch.setAccelerator(javax.swing.KeyStroke.getKeyStroke(70, java.awt.event.KeyEvent.ALT_MASK, false));
    menuSearch.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){menuSearchActionPerformed(e);}});
  } //jbInit

  /* Manual initialization adapted from EDU.auburn.gui.GraphWindow.construct().
   */
  private void mInit()
  {
    // set window icon
    setIconImage(appIcon.getImage());
    // set target language
    targetLanguageEdit.setText(getModel().getTargetLanguage());
    // graph defaults
    Node.defaults.setBoundingBox(20, 20, 20);
    // additional controls for in the control panel
    // mouse action button group
    ButtonGroup mouseActions = new ButtonGroup();
    mouseActions.add(createConceptRadio);
    mouseActions.add(createSolRoleRadio);
    mouseActions.add(createCompositeRadio);
    mouseActions.add(createRoleRadio);
    mouseActions.add(createPublishedRadio);
    mouseActions.add(createFeatReposRadio);
    mouseActions.add(createLinkRadio);
    mouseActions.add(selectRadio);
    // hide viewport angle component
    viewAnglePanel.setVisible(false);
    // hide log panel
    logPanel.setVisible(false);
    // set layout algorithm
    layoutAlg_ = new ModelSpring();
    // enable log by default
    logger_ = this;
  } //mInit

  /**
   * Flattens the model and displays it in a new window.
   *
   * @param full If true, does a full flatten, flattens one level otherwise.
   */
  private void flatten(boolean full)
  {
    FlattenVisitor flatten;

    // create flatten visitor
    flatten = new FlattenVisitor();
    flatten.setLogger(logger_);
    flatten.setFull(full);
    flatten.setErrorReporter(this);
    flatten.setUpdate(modelCanvas_);
    // flatten model to new models
    flatten.startModel(getModel());
  } //flatten

  /**
   * Flattens the model and displays it in a new window. Uses depth-first
   * strategy and delivers only first option.
   *
   * @param full If true, does a full flatten, flattens one level otherwise.
   */
  private void flattenDepthFirst(boolean full)
  {
    // create flatten visitor
    currentFlattenCount_ = 1;
    currentFlatten_ = new FlattenVisitorDepthFirst();
    currentFlatten_.setLogger(logger_);
    currentFlatten_.setFull(full);
    currentFlatten_.setErrorReporter(this);
    currentFlatten_.setUpdate(modelCanvas_);
    // flatten model to new models
    currentFlatten_.startModel(getModel());
  } //flattenDepthFirst

  /**
   * Flattens the model and displays it in a new window. Uses depth-first
   * strategy and delivers next option.
   *
   * @param full If true, does a full flatten, flattens one level otherwise.
   */
  private void nextFlattenDepthFirst(boolean full)
  {
    if (currentFlatten_ != null)
    {
      currentFlattenCount_++;
      // flatten model to new models
      // model is passed along just for null check, but a cached model is used
      // to retain consistency with previously translated models
      currentFlatten_.startModel(getModel());
    }
    else
    {
      reportError(new CoComposeException("No next flattening options"));
    }
  } //nextFlattenDepthFirst

  /**
   * Determines the concept forms for each concept in the model and displays.
   */
  private void determineForms()
  {
    DetermineFormsVisitor determineForms;

    // determineForms operation through visitor construction
    determineForms = new DetermineFormsVisitor();
    determineForms.setLogger(logger_);
    determineForms.setErrorReporter(this);
    determineForms.setUpdate(modelCanvas_);
    // determine forms to new models
    determineForms.startModel(getModel());
  } //determineForms

  /**
   * Resets the concept forms for each concept in the model and displays.
   */
  private void resetForms()
  {
    ModelVisitor resetForms;

    // resetForms operation through visitor construction
    resetForms = new ResetFormsVisitor();
    resetForms.setLogger(logger_);
    resetForms.setErrorReporter(this);
    resetForms.setUpdate(modelCanvas_);
    // reset forms
    resetForms.startModel(getModel());
  } //resetForms

  /**
   * Generates implementation for the model.
   */
  private void generateImplementation()
  {
    GenerateImplementationVisitor generateImpl;
    String language;
    Model model = getModel();

    // retrieve target language and dynamically load implementation visitor
    language = model.getTargetLanguage();
    try
    {
      generateImpl = (GenerateImplementationVisitor) Class.forName(
        GENIMPL_PATH + language + "ImplementationVisitor").newInstance();
      // generateImplementation operation through visitor construction
      generateImpl.setLogger(logger_);
      generateImpl.setErrorReporter(this);
      generateImpl.setUpdate(modelCanvas_);
      // generate implementation
      generateImpl.startModel(model);
    }
    catch (Exception exc)
    {
      reportError(new CoComposeException("Generating implementation failed: " + exc.getMessage()));
    }
  } //generateImplementation

  /**
   * Applies the entire translation process.
   */
  private void translateAll()
  {
    ModelVisitor translate;

    // translate operation through visitor construction
    translate = new TranslateVisitor();
    translate.setLogger(logger_);
    translate.setErrorReporter(this);
    translate.setUpdate(modelCanvas_);
    // apply translation process
    translate.startModel(getModel());
  } //translateAll

  /**
   * Displays processed models in new windows after visitor operation.
   *
   * @param visitor The visitor.
   */
  private void endVisit(ModelVisitor visitor)
  {
    Collection newModels;
    Model newModel;
    Point framePos = getLocation();
    int counter = 1;
    String title;

    // check visitor class
    if (visitor instanceof FlattenVisitor)
    {
      newModels = ((FlattenVisitor) visitor).getFlatModels();
      title = " - Flattened ";
    }
    else if (visitor instanceof FlattenVisitorDepthFirst)
    {
      newModels = new Vector();
      newModels.add(((FlattenVisitorDepthFirst) visitor).getFlatModel());
      title = " - Flattened ";
      counter = currentFlattenCount_;
      currentFlatten_ = ((FlattenVisitorDepthFirst) visitor).getNextFlattenVisitor();
      menuNextFlattenOption.setEnabled(currentFlatten_ != null);
    }
    else if (visitor instanceof DetermineFormsVisitor)
    {
      newModels = ((DetermineFormsVisitor) visitor).getResultingModels();
      title = " - Forms ";
    }
    else if (visitor instanceof TranslateVisitor)
    {
      newModels = ((TranslateVisitor) visitor).getTransModels();
      title = " - Option ";
    }
    else
      return;
    // display each model
    Iterator models = newModels.iterator();
    while (models.hasNext())
    {
      newModel = (Model) models.next();
      newWindow(newModel, getTitle() + title + counter);
      counter++;
    } //while
    // force garbage collection
    System.gc();
  } //endVisit

  /* Change mouse behaviour. */
  void createConceptRadio_stateChanged(ChangeEvent e)
  {
    if (createConceptRadio.isSelected())
      modelCanvas_.setMouseMode(ModelCanvas.CREATE_CONCEPTS);
  } //createConceptRadio_stateChanged

  /* Change mouse behaviour. */
  void createSolRoleRadio_stateChanged(ChangeEvent e)
  {
    if (createSolRoleRadio.isSelected())
      modelCanvas_.setMouseMode(ModelCanvas.CREATE_SOLROLE);
  } //createSolRoleRadio_stateChanged

  /* Change mouse behaviour. */
  void createCompositeRadio_stateChanged(ChangeEvent e)
  {
    if (createCompositeRadio.isSelected())
      modelCanvas_.setMouseMode(ModelCanvas.CREATE_COMPOSITES);
  } //createCompositeRadio_stateChanged

  /* Change mouse behaviour. */
  void createRoleRadio_stateChanged(ChangeEvent e)
  {
    if (createRoleRadio.isSelected())
      modelCanvas_.setMouseMode(ModelCanvas.CREATE_ROLES);
  } //createRoleRadio_stateChanged

  /* Change mouse behaviour. */
  void createPublishedRadio_stateChanged(ChangeEvent e)
  {
    if (createPublishedRadio.isSelected())
      modelCanvas_.setMouseMode(ModelCanvas.CREATE_PUBLISHED);
  } //createPublishedRadio_stateChanged

  /* Change mouse behaviour. */
  void createFeatReposRadio_stateChanged(ChangeEvent e)
  {
    if (createFeatReposRadio.isSelected())
      modelCanvas_.setMouseMode(ModelCanvas.CREATE_REPOS);
  } //createFeatReposRadio_stateChanged

  /* Change mouse behaviour. */
  void createLinkRadio_stateChanged(ChangeEvent e)
  {
    if (createLinkRadio.isSelected())
      modelCanvas_.setMouseMode(ModelCanvas.CREATE_EDGES);
  } //createLinkRadio_stateChanged

  /* Change mouse behaviour. */
  void selectRadio_stateChanged(ChangeEvent e)
  {
    if (selectRadio.isSelected())
      modelCanvas_.setMouseMode(ModelCanvas.SELECT_BOTH);
  } //selectRadio_stateChanged

  /* Zoom out. */
  void halfScaleBtn_actionPerformed(ActionEvent e)
  {
    scale_ /= 2.0;
    modelCanvas_.setScale(scale_);

    ((TitledBorder) titledBorder3.getOutsideBorder()).setTitle("Scale: " + scale_);
    // repaint border
    scalePanel.setVisible(false);
    scalePanel.setVisible(true);
  } //halfScaleBtn_actionPerformed

  /* Normal zoom. */
  void normalScaleBtn_actionPerformed(ActionEvent e)
  {
    scale_ = 1.0;
    modelCanvas_.setScale(scale_);

    ((TitledBorder) titledBorder3.getOutsideBorder()).setTitle("Scale: " + scale_);
    // repaint border
    scalePanel.setVisible(false);
    scalePanel.setVisible(true);
  } //normalScaleBtn_actionPerformed

  /* Zoom in. */
  void doubleScaleBtn_actionPerformed(ActionEvent e)
  {
    scale_ *= 2.0;
    modelCanvas_.setScale(scale_);

    ((TitledBorder) titledBorder3.getOutsideBorder()).setTitle("Scale: " + scale_);
    // repaint border
    scalePanel.setVisible(false);
    scalePanel.setVisible(true);
  } //doubleScaleBtn_actionPerformed

  /* Close the window. */
  void menuClose_actionPerformed(ActionEvent e)
  {
    dispose();
  } //menuClose_actionPerformed

  /* View XML representation of the model. */
  void menuViewXML_actionPerformed(ActionEvent e)
  {
    ByteArrayOutputStream stream = new ByteArrayOutputStream();

    try
    {
      SAXWriter xmlWriter = new SAXWriter();
      xmlWriter.setOutputStream(stream);
      getModel().toXML(xmlWriter, false, null);
      TextViewDialog viewDlg = new TextViewDialog(this, "XML source", true);
      viewDlg.setText("text/plain", stream.toString());
      viewDlg.show();
    }
    catch (SAXException exc)
    {
      reportError(new CoComposeException("An error occurred during creation of the XML document"));
    }
  } //menuViewXML_actionPerformed

  /* Select all elements in the model. */
  void menuSelectAll_actionPerformed(ActionEvent e)
  {
    modelCanvas_.selectAll();
  } //menuSelectAll_actionPerformed

  /* Deselect all elements in the model. */
  void menuDeselectAll_actionPerformed(ActionEvent e)
  {
    modelCanvas_.unselectItems();
  } //menuDeselectAll_actionPerformed

  /* Delete selected elements in the model. */
  void menuDelete_actionPerformed(ActionEvent e)
  {
    modelCanvas_.deleteSelected();
  } //menuDelete_actionPerformed

  /* Show properties dialog for selected node. */
  void menuProperties_actionPerformed(ActionEvent e)
  {
    if (modelCanvas_.getSelectedNode() != null)
    {
      modelCanvas_.setNodeProperties();
    }
    else
    {
      reportError(new CoComposeException("No concept, composite or role selected"));
    }
  } //menuProperties_actionPerformed

  /* Save the model to an xml file. */
  void menuSave_actionPerformed(ActionEvent e)
  {
    Model model = getModel();
    JFileChooser saveDlg = new JFileChooser("./models");
    DirFileFilter filter = new DirFileFilter(".xml");

    try
    {
      saveDlg.setSelectedFile(new File("./models/" + getTitle() + ".xml"));
      saveDlg.setFileFilter(filter);
      saveDlg.setDialogTitle("Save model");
      // show dialog
      if (saveDlg.showSaveDialog(this) != JFileChooser.APPROVE_OPTION)
        return;
      File file = saveDlg.getSelectedFile();
      // check if filename ends with .xml
      if (!file.getAbsolutePath().endsWith(".xml"))
      {
        file = new File(file.getAbsolutePath() + ".xml");
      }
      // check if the file already exists
      if (file.exists())
      {
        if (!askOverwrite(file.getName()))
          return;
      }
      // save to the file
      file.createNewFile();
      SAXWriter xmlWriter = new SAXWriter();
      xmlWriter.setOutputStream(new FileOutputStream(file));
      model.toXML(xmlWriter, false, file.getAbsolutePath());
      // set new model name and window title
      String name = file.getName();
      name = name.substring(0, name.length() - 4);
      model.setName(name);
      setTitle(name);
    }
    catch (SAXException saxException)
    {
      reportError(new CoComposeException("An error occurred during creation of the XML document"));
    }
    catch (Exception exception)
    {
      reportError(new CoComposeException("It appears your VM does not allow file saving"));
    }
  } //menuSave_actionPerformed

  /* Save the selected Composite in the repository. */
  void menuRepos_actionPerformed(ActionEvent e)
  {
    CompositeType compositeType = new CompositeType();

    // check for valid selected Composite
    Node thisNode = modelCanvas_.getSelectedNode();
    if (!(thisNode instanceof EDU.utwente.CoCompose.lang.Composite))
    {
      reportError(new CoComposeException("No composite selected"));
      return;
    }

    // selected node is a Composite, try copying
    try
    {
      compositeType.copyAttributes((EDU.utwente.CoCompose.lang.Composite) thisNode);
    }
    catch (CoComposeException exc)
    {
      reportError(new CoComposeException(
        "Invalid Composite type: " + exc.getMessage(), exc));
      return;
    }

    // copy made, now save it to file
    JFileChooser saveDlg = new JFileChooser("./repository");
    DirFileFilter filter = new DirFileFilter(".xml");
    try
    {
      saveDlg.setSelectedFile(new File("./repository/" + compositeType.getType() + ".xml"));
      saveDlg.setFileFilter(filter);
      saveDlg.setDialogTitle("Save composite to repository");
      // show dialog
      if (saveDlg.showSaveDialog(this) != JFileChooser.APPROVE_OPTION)
        return;
      File file = saveDlg.getSelectedFile();
      // check if filename ends with .xml
      if (!file.getAbsolutePath().endsWith(".xml"))
      {
        file = new File(file.getAbsolutePath() + ".xml");
      }
      // check if the file already exists
      if (file.exists())
      {
        if (!askOverwrite(file.getName()))
          return;
      }
      // save to the file
      file.createNewFile();
      SAXWriter xmlWriter = new SAXWriter();
      xmlWriter.setOutputStream(new FileOutputStream(file));
      compositeType.toXML(xmlWriter, file.getAbsolutePath());
      // notify repository container, if any
      if (repository_ != null)
      {
        repository_.buildRepository();
      }
    }
    catch (SAXException saxException)
    {
      reportError(new CoComposeException("An error occurred during creation of the XML document"));
    }
    catch (UnsupportedEncodingException encException)
    {
      reportError(new CoComposeException("An error occurred during creation of the XML document"));
    }
    catch (Exception exception)
    {
      reportError(new CoComposeException("It appears your VM does not allow file saving"));
    }
  } //menuRepos_actionPerformed

  /* Turn the selected concept into a composite, if any. */
  void menuTurnToComposite_actionPerformed(ActionEvent e)
  {
    try
    {
      modelCanvas_.changeNodeClass("EDU.utwente.CoCompose.lang.Composite");
    }
    catch (CoComposeException exc)
    {
      reportError(new CoComposeException(
        "Failed to turn selected element into a composite: " +
        exc.getMessage(), exc));
    }
  } //menuTurnToComposite_actionPerformed

  /* Turn the selected composite into a concept, if any. */
  void menuTurnToConcept_actionPerformed(ActionEvent e)
  {
    try
    {
      modelCanvas_.changeNodeClass("EDU.utwente.CoCompose.lang.Concept");
    }
    catch (CoComposeException exc)
    {
      reportError(new CoComposeException(
        "Failed to turn selected element into a concept: " +
        exc.getMessage(), exc));
    }
  } //menuTurnToConcept_actionPerformed

  /* Turn the selected primitive into a composite from the repository,
   * if any. */
  void menuTurnToRepos_actionPerformed(ActionEvent e)
  {
    EDU.utwente.CoCompose.lang.Composite newReposComposite =
      repository_.getSelectedComposite();
    // if no composite selected, return
    if (newReposComposite == null)
    {
      reportError(new CoComposeException(
        "No composite selected from the repository", null, "Error"));
      return;
    }
    // replace node
    try
    {
      modelCanvas_.changeNode(newReposComposite);
    }
    catch (CoComposeException exc)
    {
      exc = new CoComposeException("Failed to turn selected element into a composite: " +
        exc.getMessage(), exc);
      reportError(exc);
    }
  } //menuTurnToRepos_actionPerformed

  /* Update all composites in the model using the composite type from the
   * repository, if any. */
  void menuUpdateComposites_actionPerformed(ActionEvent e)
  {
    EDU.utwente.CoCompose.lang.Composite newReposComposite =
      repository_.getSelectedComposite();
    // if no composite selected, return
    if (newReposComposite == null)
    {
      reportError(new CoComposeException(
        "No composite selected from the repository", null, "Error"));
      return;
    }
    // replace node
    try
    {
      modelCanvas_.getModel().refreshFromComposite(newReposComposite);
      modelCanvas_.repaint();
    }
    catch (CoComposeException exc)
    {
      exc = new CoComposeException("Failed to turn selected element into a composite: " +
        exc.getMessage(), exc);
      reportError(exc);
    }
  } //menuUpdateComposites_actionPerformed

  /* Turn the selected composite into a role concept, if any. */
  void menuTurnToSolutionRole_actionPerformed(ActionEvent e)
  {
    try
    {
      modelCanvas_.changeNodeClass("EDU.utwente.CoCompose.lang.SolutionRole");
    }
    catch (CoComposeException exc)
    {
      exc = new CoComposeException("Failed to turn selected element into a solution role: " +
        exc.getMessage(), exc);
      reportError(exc);
    }
  } //menuTurnToSolutionRole_actionPerformed

  /* Applies automatic layout. Adapted from GraphWindow.applyAlgorithm_. */
  void menuScatter_itemStateChanged(ItemEvent e)
  {
    if (menuScatter.getState())
    {
      try
      {
        modelSpring_.start(getModel(), modelCanvas_);
      }
      catch (CoComposeException exc)
      {
        reportError(exc);
      }
    }
    else
    {
      modelSpring_.stop();
    }
  } //menuScatter_itemStateChanged

  /* Flattens the model and displays it in a new window. */
  void menuFlatten_actionPerformed(ActionEvent e)
  {
    /* showing the pane and starting a thread kills the MacOS X VM, so don't */
    /* update on that: thread now waits 1 second for main thread to recover. */
    menuShowLog.setState(menuLogToggle.getState());
    // start full flatten
    flatten(true);
  } //menuFlatten_actionPerformed

  /* Flattens the model by one level and displays it in a new window. */
  void menuFlattenOne_actionPerformed(ActionEvent e)
  {
    /* showing the pane and starting a thread kills the MacOS X VM, so don't */
    /* update on that: thread now waits 1 second for main thread to recover. */
    menuShowLog.setState(menuLogToggle.getState());
    // start one level flatten
    flatten(false);
  } //menuFlattenOne_actionPerformed
  
  /* Flattens the model using depth-first strategy and returns first option. */
  void menuFlattenDepthFirstActionPerformed(ActionEvent e)
  {
    /* showing the pane and starting a thread kills the MacOS X VM, so don't */
    /* update on that: thread now waits 1 second for main thread to recover. */
    menuShowLog.setState(menuLogToggle.getState());
    flattenDepthFirst(true);
  } //menuFlattenDepthFirstActionPerformed

  /* Calculates and returns the next depth-first option. */
  void menuNextFlattenOptionActionPerformed(ActionEvent e)
  {
    menuShowLog.setState(menuLogToggle.getState());
    nextFlattenDepthFirst(true);
  } //menuNextFlattenOptionActionPerformed

  /* Determines the concept forms for each concept and displays them. */
  void menuDetermine_actionPerformed(ActionEvent e)
  {
    /* showing the pane and starting a thread kills the MacOS X VM, so don't
    /* update on that: thread now waits 1 second for main thread to recover. */
    menuShowLog.setState(menuLogToggle.getState());
    determineForms();
  } //menuDetermine_actionPerformed

  /* Starts the reset concept forms procedure. */
  void menuResetForms_actionPerformed(ActionEvent e)
  {
    /* showing the pane and starting a thread kills the MacOS X VM, so don't */
    /* update on that: thread now waits 1 second for main thread to recover. */
    menuShowLog.setState(menuLogToggle.getState());
    resetForms();
  } //menuResetForms_actionPerformed

  /* Generates implementation in a target language for the model. */
  void menuGenerate_actionPerformed(ActionEvent e)
  {
    /* showing the pane and starting a thread kills the MacOS X VM, so don't */
    /* update on that: thread now waits 1 second for main thread to recover. */
    menuShowLog.setState(menuLogToggle.getState());
    generateImplementation();
  } //menuGenerate_actionPerformed

  /* Applies all translation process steps. */
  void menuTranslateAll_actionPerformed(ActionEvent e)
  {
    /* showing the pane and starting a thread kills the MacOS X VM, so don't */
    /* update on that: thread now waits 1 second for main thread to recover. */
    menuShowLog.setState(menuLogToggle.getState());
    translateAll();
  } //menuTranslateAll_actionPerformed

  /* Edit language button pressed. */
  void editButton_actionPerformed(ActionEvent e)
  {
    modelPropertiesMenuItem_actionPerformed(e);
  } //editButton_actionPerformed

  /* Show or hide viewing angle control panel. */
  void menuShowAngle_itemStateChanged(ItemEvent e)
  {
    viewAnglePanel.setVisible(menuShowAngle.getState());
  } //menuShowAngle_itemStateChanged

  /* Show or hide menu panel. */
  void menuShowJMenu_itemStateChanged(ItemEvent e)
  {
    menuPanel.setVisible(menuShowJMenu.getState());
  } //menuShowJMenu_itemStateChanged

  /* Show or hide mouse action panel. */
  void menuShowMouse_itemStateChanged(ItemEvent e)
  {
    mouseActionPanel.setVisible(menuShowMouse.getState());
  } //menuShowMouse_itemStateChanged

  /* Show or hide viewing offset control panel. */
  void menuShowOffset_itemStateChanged(ItemEvent e)
  {
    viewOffsetPanel.setVisible(menuShowOffset.getState());
  } //menuShowOffset_itemStateChanged

  /* Show or hide viewing scale control panel. */
  void menuShowScale_itemStateChanged(ItemEvent e)
  {
    scalePanel.setVisible(menuShowScale.getState());
  } //menuShowScale_itemStateChanged

  /* Show or hide target language panel. */
  void menuShowLang_itemStateChanged(ItemEvent e)
  {
    targetLangPanel.setVisible(menuShowLang.getState());
  } //menuShowLang_itemStateChanged

  /* The state of the AngleControlPanel has changed. */
  void viewAnglePanel_stateChanged(ChangeEvent e)
  {
    DPoint angles = viewAnglePanel.getAngles();
    modelCanvas_.setWireframe(viewAnglePanel.isDragging());
    modelCanvas_.setViewAngles(angles.x, angles.y);
  } //viewAnglePanel_stateChanged

  /* The state of the ViewPortPanel has changed. */
  void viewOffsetPanel_stateChanged(ChangeEvent e)
  {
    DPoint offset = viewOffsetPanel.getOffset();
    modelCanvas_.setWireframe(viewOffsetPanel.isDragging());
    viewingPanel.scrollTo(offset.x, offset.y);
  } //viewOffsetPanel_stateChanged

  /* The state of the model viewing panel has changed. */
  void viewingPanel_stateChanged(ChangeEvent e)
  {
    Node selectedNode;
    Point selectedLink;
    boolean selected, nodeSelected, compositeSelected;

    // update viewing controls
    viewOffsetPanel.setPortSize(viewingPanel.getPortSize());
    viewOffsetPanel.setContentSize(viewingPanel.getContentSize());
    viewOffsetPanel.setOffset(viewingPanel.getOffset());
    // update undo/redo state
    menuUndo.setEnabled(modelCanvas_.canUndo());
    menuRedo.setEnabled(modelCanvas_.canRedo());
    // update selection-based controls
    selectedNode = modelCanvas_.getSelectedNode();
    selectedLink = modelCanvas_.getSelectedLink();
    nodeSelected = (selectedNode != null);
    selected = nodeSelected || (selectedLink != null);
    if (nodeSelected)
      compositeSelected = selectedNode instanceof EDU.utwente.CoCompose.lang.Composite;
    else
      compositeSelected = false;
    menuDelete.setEnabled(selected);
    menuDeselectAll.setEnabled(selected);
    menuTurnToConcept.setEnabled(nodeSelected);
    menuTurnToSolutionRole.setEnabled(nodeSelected);
    menuTurnToComposite.setEnabled(nodeSelected);
    menuTurnToRepos.setEnabled(nodeSelected);
    menuProperties.setEnabled(nodeSelected);
    menuRepos.setEnabled(compositeSelected);
    reposBtn.setEnabled(compositeSelected);
    // reset flatten next option
    menuNextFlattenOption.setEnabled(false);
  } //viewingPanel__stateChanged

  /* Mouse button released over canvas. */
  void modelCanvas__mouseReleased(MouseEvent e)
  {
    // handle popup
    if (e.isPopupTrigger() && (modelCanvas_.getSelectedNode() != null))
    {
      // popup menu
      popupMenu1.show(modelCanvas_, e.getX(), e.getY());
    }
  } //modelCanvas__mouseReleased

  /* Save model as Encapsulated PostScript. */
  void menuEPS_actionPerformed(ActionEvent e)
  {
    JFileChooser saveDlg = new JFileChooser();
    DirFileFilter filter = new DirFileFilter(".eps");
    try
    {
      saveDlg.setFileFilter(filter);
      saveDlg.setDialogTitle("Save model as EPS");
      // show dialog
      if (saveDlg.showSaveDialog(this) != JFileChooser.APPROVE_OPTION)
        return;
      File file = saveDlg.getSelectedFile();
      // check if filename ends with .eps
      if (!file.getAbsolutePath().endsWith(".eps"))
      {
        file = new File(file.getAbsolutePath() + ".eps");
      }
      // check if the file already exists
      if (file.exists())
      {
        if (!askOverwrite(file.getName()))
          return;
      }
      // save to the file
      String postscript = modelCanvas_.toEPS();
      file.createNewFile();
      FileOutputStream out = new FileOutputStream(file);
      out.write(postscript.getBytes());
      out.close();
    }
    catch (Exception exc)
    {
      CoComposeException exc2 = new CoComposeException(
        "It appears your VM does not allow file saving: " +
        exc.getMessage());
      reportError(exc2);
    }
  } //menuPS_actionPerformed

  /* Shows/hides log panel. */
  void menuShowLog_itemStateChanged(ItemEvent e)
  {
    logPanel.setVisible(menuShowLog.getState());
  } //menuShowLog_itemStateChanged

  void menuLogToggle_itemStateChanged(ItemEvent e)
  {
    if (menuLogToggle.getState())
      logger_ = this;
    else
      logger_ = null;
  } //menuLogToggle_itemStateChanged

  /* Clears the logging panel. */
  void menuClearLog_actionPerformed(ActionEvent e)
  {
    log = new DefaultStyledDocument();
    logPane.setDocument(log);
  } //menuClearLog_actionPerformed

  /* Shows model properties dialog. */
  void modelPropertiesMenuItem_actionPerformed(ActionEvent e)
  {
    Model model = getModel();

    ModelPropertiesDialog dialog = new ModelPropertiesDialog(this,
      "Edit model properties", true, model);
    dialog.show();
    targetLanguageEdit.setText(model.getTargetLanguage());
  } //modelPropertiesMenuItem_actionPerformed

  /* Load model positions from another model. */
  void menuPosition_actionPerformed(ActionEvent e)
  {
    Model sourcemodel;
    PositionVisitor position;
    JFileChooser openDlg = new JFileChooser("models");
    DirFileFilter filter = new DirFileFilter(".xml");

    try
    {
      // filter files on .xml
      openDlg.setFileFilter(filter);
      openDlg.setDialogTitle("Open model");
      if (openDlg.showOpenDialog(this) != JFileChooser.APPROVE_OPTION)
        return;
      File file = openDlg.getSelectedFile();
      // filename chosen - open this file
      String uri = file.getAbsolutePath();
      try
      {
        // load source model
        String name = file.getName();
        if (name.endsWith(".xml"))
          name = name.substring(0, name.length() - 4);
        sourcemodel = new Model(uri, name);
        // now copy positions in separate thread
        position = new PositionVisitor(sourcemodel);
        position.setErrorReporter(this);
        position.setUpdate(modelCanvas_);
        position.startModel(getModel());
      }
      catch (CoComposeException exc)
      {
        exc = new CoComposeException("There was an error loading the new positions: " +
          exc.getMessage(), exc);
        reportError(exc);
      }
      modelCanvas_.repaint();
    }
    catch (Exception exc)
    {
      CoComposeException exc2 = new CoComposeException(
        "It appears your VM does not allow file loading: " +
        exc.getMessage());
      reportError(exc2);
    }
  } //menuPosition_actionPerformed

  /* Hides logging panel. */
  void popupHideLog_actionPerformed(ActionEvent e)
  {
    menuShowLog.setState(false);
  } //popupHideLog_actionPerformed

  /* Mouse button released over logging pane. */
  void logPane_mouseReleased(MouseEvent e)
  {
    // handle popup
    if (e.isPopupTrigger())
    {
      // popup menu
      logPopupMenu.show(logPane, e.getX(), e.getY());
    }
  } //logPane_mouseReleased

  /* Handle window closed event. */
  void this_windowClosed(WindowEvent e)
  {
    // stop ModelSpring algorithm
    modelSpring_.stop();
  } //this_windowClosed

  /* Performs undo on model canvas. */
  void menuUndo_actionPerformed(ActionEvent e)
  {
    // disable scatter and undo
    menuScatter.setState(false);
    modelCanvas_.undo();
  } //menuUndo_actionPerformed

  /* Performs redo on model canvas. */
  void menuRedo_actionPerformed(ActionEvent e)
  {
    // disable scatter and redo
    menuScatter.setState(false);
    modelCanvas_.redo();
  } //menuRedo_actionPerformed
  
  /* Opens a search dialog */
  void menuSearchActionPerformed(ActionEvent e)
  {
    SearchDialog dlg = new SearchDialog(this, false, modelCanvas_);
    dlg.show();
  } //menuSearchActionPerformed

  //////////////////////////////////////////////////////////////////////////////
  // Protected methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Overridden event processing to link close event to button.
   *
   * @param e The window event.
   */
  protected void processWindowEvent(WindowEvent e)
  {
    if(e.getID() == WindowEvent.WINDOW_CLOSING)
    {
      this_windowClosed(e);
    }
    super.processWindowEvent(e);
  } //processWindowEvent

  /**
   * Creates and shows a new model window
   * @param model - the model to display
   * @param title - the window title
   */
  protected void newWindow(Model model, String title)
  {
    Point framePos = getLocation();
    framePos.translate(20, 20);
    ModelFrame model_window = new ModelFrame(model);
    model_window.setLocation(framePos.x, framePos.y);
    model_window.setTitle(title);
    model_window.setRepositoryContainer(repository_);
    model_window.pack();
    model_window.setSize(getSize());
    model_window.show();
  } //newWindow

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Gets the model being edited in this window.
   *
   * @return The model being edited.
   */
  public Model getModel()
  {
    return modelCanvas_.getModel();
  } //getModel

  /**
   * Reports an error.
   *
   * @param error - the error to report
   */
  public void reportError(CoComposeException error)
  {
    AbstractConcept concept = error.getConcept();
    String type = error.getType();
    String message = error.getMessage();
    if (concept != null)
    {
      concept.setFillColor(Color.red);
      if (!concept.getModel().equals(getModel()))
      { // create window for new model
        newWindow(concept.getModel(), concept.getModel().getName() + ": " + message);
      }
      else
      {
        modelCanvas_.repaint();
      }
    }
    MessageDialog errDlg = new MessageDialog(this, type, true);
    // switch type
    if (type.equals("Error"))
      errDlg.setIcon(MessageDialog.ICON_ERROR);
    else if (type.equals("Warning"))
      errDlg.setIcon(MessageDialog.ICON_WARNING);
    else if (type.equals("Info"))
      errDlg.setIcon(MessageDialog.ICON_INFO);
    errDlg.setMessage(message);
    errDlg.show();
  } //reportError

  /**
   * Accepts generic notification message.
   *
   * @param sender The message sender.
   */
  public void notify(Object sender)
  {
    // Dispatch sender
    if (sender instanceof EDU.utwente.CoCompose.lang.ModelVisitor)
      endVisit((ModelVisitor) sender);
  } //notify

  /**
   * Shows a file overwrite dialog and returns true when overwriting is ok.
   *
   * @param filename The name of the file that is about to be overwritten.
   * @return True if and only if the file may be overwritten.
   */
  public boolean askOverwrite(String filename)
  {
    MessageDialog msg = new MessageDialog(this, "Warning", true);
    int[] buttons = {MessageDialog.BUTTON_YES, MessageDialog.BUTTON_NO};
    msg.setButtons(buttons);
    msg.setIcon(MessageDialog.ICON_WARNING);
    msg.setMessage("The file " + filename + " already exists. Overwrite?");
    msg.show();
    if (msg.getPressedButton() == MessageDialog.BUTTON_YES)
      return true;
    else
      return false;
  } //askOverwrite

  /**
   * Sets the repository container, so when this object changes the repository
   * it will notify the repository container.
   *
   * @param repositoryContainer The object that contains the repository tree.
   */
  public void setRepositoryContainer(RepositoryContainer repositoryContainer)
  {
    repository_ = repositoryContainer;
    modelCanvas_.setRepositoryContainer(repositoryContainer);
  } //setRepositoryContainer

  /**
   * Logs a message and forces window update.
   *
   * @param message The message to log.
   */
  public void log(String message)
  {
    Date logDate = new Date();
    logBuffer_.append(logFormatter_.format(logDate) + logIndent_ + message + '\n');
    // only display log updates occasionally to save processing power
    if (logDate.before(oldLogDate_) && (logLevel_ > 0))
      return;
    // update log panel every 0.25 seconds
    oldLogDate_ = new Date(logDate.getTime() + 250);
    try
    {
      log.insertString(log.getLength(), logBuffer_.toString(), null);
      logPane.setCaretPosition(log.getLength());
      logBuffer_.delete(0, logBuffer_.length());
    }
    catch (BadLocationException e)
    { // should never happen
      e.printStackTrace();
    }
  } //log

  /**
   * Increases the log level for the next messages.
   */
  public void incLogLevel()
  {
    logLevel_++;
    logIndent_ += "  ";
  } //incLogLevel

  /**
   * Decreases the log level for the next messages.
   */
  public void decLogLevel()
  {
    if (logLevel_ > 0)
    {
      logLevel_--;
      logIndent_ = logIndent_.substring(0, logIndent_.length() - 2);
    }
  } //decLogLevel
} //class ModelFrame
