/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.translate;

import EDU.utwente.CoCompose.gui.ModelUpdate;
import EDU.utwente.CoCompose.lang.*;
import EDU.utwente.CoCompose.util.*;

/**
 * Determines the concept forms of all concepts in a model.
 *
 * @author Dennis Wagelaar
 * @version 0.1.10, 22 July 2003
 */

public class ResetFormsVisitor extends ModelVisitor
{
  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a ModelVisitor.
   */
  public ResetFormsVisitor()
  {
  } //ResetFormsVisitor

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Resets the concept forms for each concept in the model.
   *
   * @param model The model to visit.
   * @throws CoComposeException if not all concept forms could be reset.
   */
  public void visitModel(Model model) throws CoComposeException
  {
    Concept concept;
    ModelUpdate update = getUpdate();

    if (log != null)
    {
      log.log("-- Started resetting concept forms for " + model.getName());
      log.incLogLevel();
    }
    try
    {
      // reset forms for each concept
      for (concept = model.firstConcept(); concept != null;
        concept = model.nextConcept(concept))
      {
        concept.accept(this);
      }
    }
    finally
    {
      if (log != null)
      {
        log.decLogLevel();
        log.log("-- Completed resetting concept forms for " + model.getName());
      }
      // refresh screen
      if (update != null)
        update.update(false);
    }
  } //visitModel

  /**
   * Resets the possible concept forms for this concept.
   *
   * @param concept The concept to determine the forms for.
   * @throws CoComposeException if the concept forms could not be reset.
   */
  public void visitConcept(Concept concept) throws CoComposeException
  {
    if (log != null)
    {
      log.log("Resetting concept form for " + concept.getName());
      log.incLogLevel();
    }
    try
    {
      concept.setForm("");
    }
    finally
    {
      if (log != null)
        log.decLogLevel();
    }
  } //visitConcept
} //class ResetFormsVisitor
