/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.util;

import java.io.*;

/**
 * FileFilter for filtering directories and files.
 *
 * @author Dennis Wagelaar
 * @version 0.1.10, 22 July 2003
 */

public class DirFileFilter extends javax.swing.filechooser.FileFilter
  implements java.io.FileFilter
{
  private boolean acceptDirs_ = true;
  private boolean acceptFiles_ = true;
  private String endsWith_ = "";

  /**
   * Constructs a DirFileFileFilter.
   *
   * @param endsWith The file extension including dot (e.g. ".xml").
   */
  public DirFileFilter(String endsWith)
  {
    super();
    endsWith_ = endsWith;
  } //DirFileFilter

  /**
   * Gets the acceptDirs attribute.
   *
   * @return Whether to accept directories.
   */
  public boolean getAcceptDirs()
  {
    return acceptDirs_;
  } //getAcceptDirs

  /**
   * Sets the acceptDirs attribute.
   *
   * @param acceptDirs Whether to accept directories.
   */
  public void setAcceptDirs(boolean acceptDirs)
  {
    acceptDirs_ = acceptDirs;
  } //setAcceptDirs

  /**
   * Gets the acceptFiles attribute.
   *
   * @return Whether to accept XML files.
   */
  public boolean getAcceptFiles()
  {
    return acceptFiles_;
  } //getAcceptFiles

  /**
   * Sets the acceptFiles attribute.
   *
   * @param acceptFiles Whether to accept XML files.
   */
  public void setAcceptFiles(boolean acceptFiles)
  {
    acceptFiles_ = acceptFiles;
  } //setAcceptFiles

  /**
   * Gets the file extension to check on.
   *
   * @return The file extension including dot (e.g. ".xml").
   */
  public String getEndsWith()
  {
    return endsWith_;
  } //getEndsWith

  /**
   * Sets the file extension to check on.
   *
   * @param endsWith The file extension including dot (e.g. ".xml").
   */
  public void setEndsWith(String endsWith)
  {
    endsWith_ = endsWith;
  } //setEndsWith

  /**
   * Whether the given file is accepted by this filter.
   *
   * @param f The file/path to check.
   * @return True if and only if the file f should be accepted.
   */
  public boolean accept(File f)
  {
    return ( ((getAcceptDirs()) && (f.isDirectory())) ||
             ((getAcceptFiles()) && (f.getAbsolutePath().endsWith(getEndsWith()))) );
  } //accept

  /**
   * The description of this filter.
   *
   * @return The description.
   */
  public String getDescription()
  {
    String type = getEndsWith().substring(1).toUpperCase();

    return(type + " files (*" + getEndsWith() + ")");
  } //getDescription
} //class DirFileFilter
